var baseDirectory = "downloads";
var cacheRetainDays = 7;
var prtVersion = "v3.12";
var DownloadNetworkType;
(function (DownloadNetworkType) {
    DownloadNetworkType[DownloadNetworkType["CELLULAR"] = 0] = "CELLULAR";
    DownloadNetworkType[DownloadNetworkType["WIFI"] = 1] = "WIFI";
    DownloadNetworkType[DownloadNetworkType["ALL"] = 2] = "ALL";
})(DownloadNetworkType || (DownloadNetworkType = {}));

var korbytStaticContent = [
    '/fonts/',
    '/images/',
    '/img/',
    '/jClocksGMT/',
    '/builder/'
];
var ResourceManager = /** @class */ (function () {
    function ResourceManager() {
    }
    ResourceManager.initializer = function () {
        ResourceManager.downloadPossible = tizen.systeminfo.getCapability('http://tizen.org/feature/download');
        _klogger.log("downloadPossible: " + ResourceManager.downloadPossible);
        ResourceManager.dlPending = [];
        ResourceManager.staticCache = {};
        ResourceManager.isDownloading = false;
        ResourceManager.cacheCleanTimer = null;
        ResourceManager.new_etags = {};
        ResourceManager.current_etags = {};
        ResourceManager.FullDownloadPath = localStorage.getItem('fullDownloadPath');
        ResourceManager.sem = semaphore(15);
    };
    Object.defineProperty(ResourceManager, "FullDownloadPath", {
        get: function () {
            return ResourceManager.fullDownloadPath;
        },
        set: function (path) {
            if (ResourceManager.fullDownloadPath !== path) {
                ResourceManager.fullDownloadPath = path;
                SourcePatcher.baseUri = ResourceManager.fullDownloadPath;
                localStorage.setItem('fullDownloadPath', ResourceManager.fullDownloadPath);
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ResourceManager, "PlayerSplashFullPath", {
        get: function () {
            return ResourceManager.playerSplashFullPath;
        },
        set: function (path) {
            if (ResourceManager.playerSplashFullPath !== path) {
                ResourceManager.playerSplashFullPath = path;
                localStorage.setItem('playerSplashFullPath', ResourceManager.playerSplashFullPath);
            }
        },
        enumerable: false,
        configurable: true
    });
    ResourceManager.isStaticContent = function (fn) {
        if (typeof fn === 'string') {
            for (var i_1 = 0; i_1 < korbytStaticContent.length; i_1++) {
                if (fn.startsWith(korbytStaticContent[i_1])) {
                    return true;
                }
            }
        }
        return false;
    };
    ResourceManager.getResources = function (cleanCache, resources, showProgress, callback) {
        ResourceManager.startTime = new Date();
        clearTimeout(ResourceManager.cacheCleanTimer);
        try {
            if (Array.isArray(resources) && resources.length > 0) {
                ResourceManager.isDownloading = true;
                showProgress = true;
                ResourceManager.countFilesDownloaded = 0;
                // always include the manifest files in the keeper list.
                ResourceManager.resourceKeeper = ResourceManager.manifestList.slice();
                resources.forEach(function (file) {
                    // append .HTML to Layouts, Messages, and Urgent Messages.
                    if (file.startsWith('/embed/') || file.startsWith('/message/') || file.startsWith('/urgent/')) {
                        file = file + '.html';
                    }
                    // Add each file to the keeper list
                    if (ResourceManager.resourceKeeper.indexOf(file) < 0) {
                        ResourceManager.resourceKeeper.push(file);
                    }
                });
                // ResourceManager.resourceKeeper.push('/tizencached.html');
                ResourceManager.cleanCacheProcess = cleanCache;
                ResourceManager.dlPending = ResourceManager.dlPending.concat(resources.slice());
                // get a reference to the caching directory.
                return tizen.filesystem.resolve(baseDirectory, function (dir) {
                    ResourceManager.dlDirectory = dir;
                    if (ResourceManager.finalCallback != null) {
                        _klogger.error("ResourceManager.finalCallback is already set?");
                    }
                    ResourceManager.finalCallback = callback;
                    // return setTimeout(ResourceManager.downloadNext, 0);
                    if (showProgress) {
                        localStorage.setItem('KorbytDownloadProgress', 'starting');
                        PlayerHost.showResourceManagerProgress();
                    }
                    return ResourceManager.downloadNext();
                }, function (err) {
                    ResourceManager.isDownloading = false;
                    _klogger.error(err);
                    return callback(err);
                });
            }
        }
        catch (err) {
            _klogger.error(err);
            return callback(err);
        }
    };
    ResourceManager.deleteDownloadedFile = function (filePath, callback) {
        return tizen.filesystem.resolve(ResourceManager.getDirectory(filePath), function (dir) {
            dir.deleteFile(filePath, callback, callback);
        }, function (err) {
            _klogger.debug(err);
            return callback();
        });
    };
    ResourceManager.start = function (tenantId, showProgress, callback) {
        try {
            clearTimeout(ResourceManager.cacheCleanTimer);
            ResourceManager.dlPending = [];
            ResourceManager.manifestList = [];
            if (ResourceManager.downloadPossible) {
                return ResourceManager.deleteDownloadedFile('downloads/prt/' + prtVersion + '/manifest.json', function () {
                    ResourceManager.forceDownloads = true;
                    return tizen.filesystem.resolve('downloads/etags.json', function (etagsFile) {
                        try {
                            return etagsFile.readAsText(function (etags) {
                                try {
                                    if (typeof etags === 'string') {
                                        etags = JSON.parse(etags);
                                    }
                                }
                                catch (err) {
                                    console.log(etags);
                                    return ResourceManager.downloadManifest(tenantId, showProgress, function (err) {
                                        return callback(err);
                                    });
                                }
                                _klogger.debug("ETAGS CONTENTS: " + JSON.stringify(etags));
                                ResourceManager.current_etags = etags;
                                return ResourceManager.downloadManifest(tenantId, showProgress, function (err) {
                                    return callback(err);
                                });
                            });
                        }
                        catch (err) {
                            _klogger.error(err);
                            console.log('ERROR Parsing ETAGS');
                            return ResourceManager.downloadManifest(tenantId, showProgress, function (err) {
                                return callback(err);
                            });
                        }
                    }, function (err) {
                        console.log('NO ETAGS.JSON');
                        return ResourceManager.downloadManifest(tenantId, showProgress, function (err) {
                            return callback(err);
                        });
                    });
                });
            }
        }
        catch (err) {
            _klogger.error(err);
            return callback(err);
        }
    };
    ResourceManager.downloadManifest = function (tenantId, showProgress, callback) {
        ResourceManager.forceDownloads = true;
        return ResourceManager.getResources(false, ['/prt/' + prtVersion + '/manifest.json'], showProgress, function () {
            try {
                ResourceManager.forceDownloads = false;
                return tizen.filesystem.resolve('downloads/prt/' + prtVersion + '/manifest.json', function (manifestFile) {
                    try {
                        return manifestFile.readAsText(function (manifest) {
                            if (typeof manifest === 'string') {
                                manifest = JSON.parse(manifest);
                            }
                            if (!Array.isArray(manifest.resources) || manifest.resources.length < 10) {
                                throw new Error('player runtime manifest is invalid or empty');
                            }
                            _klogger.info("MANIFEST CONTENTS: " + JSON.stringify(manifest));
                            ResourceManager.resourceKeeper = [];
                            var resources = [];

                            for (var ndx = 0; ndx < manifest.resources.length; ndx++) {
                                var fileName = manifest.resources[ndx].fn;
                                resources.push(fileName);
                            }
                            ResourceManager.manifestList = resources;
                            return ResourceManager.getResources(false, resources, showProgress, callback);
                        }, function (err) {
                            _klogger.error(err);
                            return callback(err);
                        });
                    }
                    catch (err) {
                        _klogger.error(err);
                        return callback(err);
                    }
                }, function (err) {
                    _klogger.error(err);
                    return callback(err);
                });
            }
            catch (err) {
                _klogger.error(err);
                return callback(err);
            }
        });
    };
    ResourceManager.getFilename = function (path) {
        try {
            return path.split('\\').pop().split('/').pop();
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.getDirectory = function (path) {
        try {
            return path.substring(0, path.lastIndexOf("/"));
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.downloadCacheFile = function (dlResource) {
        try {
            if (!dlResource.serverUrl) {
                throw new Error('downloadCacheFile(): dlResource.serverUrl is not defined');
            }
            var url_1 = dlResource.serverUrl + ("?pid=" + main.pal.playerID);
            var headers = {};
            if (!ResourceManager.forceDownloads && dlResource.lastModifiedDate) {
                headers['If-Modified-Since'] = dlResource.lastModifiedDate;
            }
            if (!ResourceManager.forceDownloads && ResourceManager.current_etags[dlResource.serverUrl]) {
                headers['If-None-Match'] = ResourceManager.current_etags[dlResource.serverUrl];
            }
            headers['Cookie'] = "tx=" + main.pal.provisioning.tenant_number;
            dlResource.tizenRequest = new tizen.DownloadRequest(url_1, dlResource.localFullDirectory, dlResource.fileName, 'ALL', headers);
            dlResource.downloadId = tizen.download.start(dlResource.tizenRequest, {
                onprogress: function (id, receivedSize, totalSize) {
                    _klogger.debug("##### ResourceManager.onProgress - " + id + ": " + receivedSize + " of " + totalSize);
                    clearTimeout(dlResource.dlTimeout);
                    dlResource.dlTimeout = setTimeout(ResourceManager.downloadOnTimeout, 60 * 1000);
                },
                onpaused: function (id) {
                    _klogger.info("##### ResourceManager.onPaused - download paused: " + id);
                    clearTimeout(dlResource.dlTimeout);
                    dlResource.dlTimeout = setTimeout(ResourceManager.downloadOnTimeout, 60 * 1000);
                },
                oncanceled: function (id) {
                    dlResource.tizenRequest = null;
                    clearTimeout(dlResource.dlTimeout);
                    return ResourceManager.onDownloadFinish();
                },
                oncompleted: function (id, fullPath) {
                    try {
                        _klogger.info("##### ResourceManager.onCompleted - download completed: " + id + " " + fullPath);
                        if (dlResource) {
                            clearTimeout(dlResource.dlTimeout);
                            dlResource.dlTimeout = null;
                            if (!ResourceManager.FullDownloadPath) {
                                ResourceManager.FullDownloadPath = fullPath.substr(0, fullPath.length - dlResource.serverPath.length);
                                SourcePatcher.baseUri = ResourceManager.FullDownloadPath;
                                _klogger.info("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FullDownloadPath === " + ResourceManager.FullDownloadPath);
                            }
                            if (dlResource.serverPath.startsWith('/embed/') || dlResource.serverPath.startsWith('/message/') || dlResource.serverPath.startsWith('/urgent/')) {
                                dlResource.fileName = dlResource.fileName + '.html';
                            }
                            var orgName = ResourceManager.getFilename(dlResource.fileName);
                            var newName = ResourceManager.getFilename(fullPath);
                            if (ResourceManager.isStaticContent(dlResource.serverPath)) {
                                ResourceManager.staticCache[dlResource.serverPath] = true;
                            }
                            if (orgName !== newName) {
                                _klogger.debug("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ RENAME FILE " + fullPath + " ===> " + newName + " ===> " + orgName);
                                return ResourceManager.renameFile(fullPath, orgName, function (fileName) {
                                    return ResourceManager.postFileWork(fileName);
                                });
                            }
                            else {
                                return ResourceManager.postFileWork(fullPath);
                            }
                        }
                    }
                    catch (err) {
                        _klogger.error(err);
                    }
                },
                onfailed: function (id, error) {
                    dlResource.tizenRequest = null;
                    try {
                        tizen.download.abandon(id);
                    } catch (e) {
                        console.error('Error cancelling download:', e);
                    }
                    clearTimeout(dlResource.dlTimeout);
                    if (error.message && error.message.includes('HTTP status: 410')) {
                        _klogger.error("#### ResourceManager - Received 410 status code: " + id + " => " + error);
                        main.deprovisioned();
                        main.setNextHeartbeat(true);
                        return;
                    }
                    if (error.code !== -1) {
                        _klogger.error("##### ResourceManager - download failed: " + id + " => " + error);
                    }
                    else {
                        if (error.message && error.message.indexOf('HTTP status: 304') < 0) {
                            _klogger.debug("##### ResourceManager - download failed: " + id + " => " + error);
                        }
                        else {
                            console.info(url_1 + ": 304 " + error.message);
                            if (ResourceManager.isStaticContent(dlResource.serverPath)) {
                                ResourceManager.staticCache[dlResource.serverPath] = true;
                            }
                        }
                    }

                    return ResourceManager.postFileWorkDone();

                }
            });
            dlResource.dlTimeout = setTimeout(function () {
                return ResourceManager.downloadOnTimeout(dlResource);
            }, 60 * 1000);
            _klogger.debug("##### ResourceManager - started download #" + dlResource.downloadId + " of '" + url_1 + "', remaining files " + ResourceManager.dlPending.length);
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.downloadOnTimeout = function (dlResource) {
        try {
            if (dlResource) {
                dlResource.tizenRequest = null;
                _klogger.info("##### ResourceManager.onCompleted - DOWNLOAD TIMEOUT: " + dlResource.serverPath);
                tizen.download.abandon(dlResource.downloadId);
                clearTimeout(dlResource.dlTimeout);
            }
            return ResourceManager.downloadNext();
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.postFileWorkDone = function (err) {
        try {
            if (err) {
                _klogger.error(err);
            }
            // running count of files downloaded.
            ResourceManager.countFilesDownloaded++;
            // return setTimeout(ResourceManager.downloadNext, 0);
            try {
                if (ResourceManager.sem.available() < 15) {
                    ResourceManager.sem.leave();
                }
            } catch (err) {
                _klogger.error("##### ResourceManager - .leave called too many times");
            }
            return ResourceManager.downloadNext();
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.postFileWork = function (fullPath) {
        try {
            _klogger.info("##### ResourceManager.postFileWork - " + fullPath);
            var ext = fullPath.substr(fullPath.lastIndexOf('.') + 1).toLowerCase();
            if (ext === 'html' || ext === 'css' || ext === 'js') {
                if (ResourceManager.getFilename(fullPath).toLowerCase() != 'playerengine.js') {
                    return tizen.filesystem.resolve(fullPath, function (file) {
                        return file.readAsText(function (fileText) {
                            try {
                                var patchedText_1 = SourcePatcher.patch(fileText, ResourceManager.FullDownloadPath);
                                if (patchedText_1 !== fileText) {
                                    _klogger.info("##### ResourceManager - write patched file " + file.fullPath);
                                    return file.openStream("w", function (fStream) {
                                        fStream.write(patchedText_1);
                                        fStream.close();
                                        return ResourceManager.postFileWorkDone();
                                    }, function (err) {
                                        return ResourceManager.postFileWorkDone(err);
                                    }, "UTF-8");
                                }
                                else {
                                    return ResourceManager.postFileWorkDone();
                                }
                            }
                            catch (err) {
                                _klogger.error(err);
                            }
                        }, function (err) {
                            return ResourceManager.postFileWorkDone(err);
                        });
                    }, function (err) {
                        return ResourceManager.postFileWorkDone(err);
                    });
                }
            }
        }
        catch (err) {
            return ResourceManager.postFileWorkDone(err);
        }
        return ResourceManager.postFileWorkDone();
    };
    ResourceManager.renameFile = function (sourcePath, fileName, callback) {
        try {
            var srcDirectory = ResourceManager.getDirectory(sourcePath);
            var fullPath_1 = srcDirectory + '/' + fileName;
            return tizen.filesystem.resolve(srcDirectory, function (srcFile) {
                return srcFile.moveTo(sourcePath, fullPath_1, true, function () {
                    _klogger.info("##### ResourceManager - FILE MOVED " + sourcePath + " ==> " + fullPath_1);
                    return callback(fullPath_1);
                }, function (err) {
                    _klogger.error("##### ResourceManager - FILE MOVED FAILED " + sourcePath + " ==> " + fullPath_1 + "\n" + err);
                    return callback(fullPath_1);
                });
            }, function (err) {
                _klogger.error(err);
                return callback(sourcePath);
            });
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.getFileInfo = function (dlResource) {
        try {
            tizen.filesystem.resolve(dlResource.localPath, function (newFile) {
                dlResource.lastModifiedDate = newFile.modified.toUTCString();
                ResourceManager.downloadCacheFile(dlResource);
            }, function (err) {
                if (err.code === 8) {
                    return ResourceManager.downloadCacheFile(dlResource);
                }
                _klogger.error(err);
            });
        }
        catch (err) {
            _klogger.error(err);
            return ResourceManager.downloadNext();
        }
    };
    ResourceManager.createCacheDirectory = function (dlResource) {
        _klogger.info("##### ResourceManager - createCacheDirectory(" + JSON.stringify(dlResource) + ")");
        try {
            ResourceManager.dlDirectory.createDirectory(dlResource.localSubDirectory);
            ResourceManager.downloadCacheFile(dlResource);
        }
        catch (err) {
            _klogger.error(err);
            return ResourceManager.downloadNext();
        }
    };
    ResourceManager.checkCacheDirectory = function (dlResource) {
        try {
            tizen.filesystem.resolve(dlResource.localFullDirectory, function (newPath) {
                return ResourceManager.getFileInfo(dlResource);
            }, function (err) {
                try {
                    if (err.code === 8) {
                        return ResourceManager.createCacheDirectory(dlResource);
                    }
                    _klogger.error(err);
                }
                catch (err) {
                    _klogger.error(err);
                }
                return ResourceManager.downloadNext();
            });
        }
        catch (err) {
            _klogger.error(err);
            return ResourceManager.downloadNext();
        }
    };
    ResourceManager.downloadNext = function () {
        try {
            if (!ResourceManager.dlPending || ResourceManager.dlPending.length < 1) {
                return ResourceManager.downloadComplete();
            }
            return tizen.systeminfo.getPropertyValue('NETWORK', function (networkInfo) {
                try {
                    if (networkInfo.networkType === 'NONE') {
                        if (ResourceManager.noNetworkLogCount <= 0) {
                            ResourceManager.noNetworkLogCount = 30;
                            _klogger.warn('##### ResourceManager - Network connection is not available. Download is not possible.');
                        }
                        else {
                            ResourceManager.noNetworkLogCount--;
                        }
                        return setTimeout(ResourceManager.downloadNext, 10 * 1000);
                    }
                    if (ResourceManager.dlPending.length > 0) {
                        ResourceManager.sem.take(function () {
                            var remaining = ResourceManager.resourceKeeper.length - ResourceManager.dlPending.length;
                            var percent = (((remaining) / ResourceManager.resourceKeeper.length) * 100).toFixed(2);
                            localStorage.setItem('KorbytDownloadProgress', remaining + "/" + ResourceManager.resourceKeeper.length + "(" + percent + "%)");
                            var dlResource = {
                                serverPath: ResourceManager.dlPending.shift(),
                            };
                            if (ResourceManager.isStaticContent(dlResource.serverPath)) {
                                if (ResourceManager.staticCache[dlResource.serverPath]) {
                                    _klogger.info(`##### ResourceManager - Already downloaded: ${dlResource.serverPath}`);
                                    return ResourceManager.onDownloadFinish(); // Skip and go onto next item
                                }
                            }
                            dlResource.serverUrl = main.pal.provisioning.cms + dlResource.serverPath;
                            dlResource.localPath = ResourceManager.dlDirectory.path + dlResource.serverPath;
                            dlResource.localFullDirectory = ResourceManager.getDirectory(dlResource.localPath);
                            dlResource.localSubDirectory = ResourceManager.getDirectory(dlResource.serverPath);
                            dlResource.fileName = ResourceManager.getFilename(dlResource.localPath);
                            // start the download process...
                            return ResourceManager.checkCacheDirectory(dlResource);
                        });
                    }
                    else {
                        return ResourceManager.downloadComplete();
                    }
                }
                catch (err) {
                    _klogger.error("##### ResourceManager - " + (err.stack || err.message));
                }
            });
        }
        catch (err) {
            _klogger.error("##### ResourceManager - " + (err.stack || err.message));
        }
        return ResourceManager.downloadNext();
    };

    ResourceManager.onDownloadFinish = function() {
        try {
            if (ResourceManager.sem.available() < 15) {
                ResourceManager.sem.leave();
            }
        } catch (err) {
            _klogger.error("##### ResourceManager - .leave called too many times");
        }
        ResourceManager.downloadNext(); // Attempt to start the next download
    };
    
    ResourceManager.cacheCleanDirectory = function (wipeYourCache, callback) {
        try {
            if (!ResourceManager.cleanerDirectoryList || ResourceManager.cleanerDirectoryList.length < 1) {
                return ResourceManager.cacheDeleteFilesList(wipeYourCache, function () {
                    _klogger.debug("##### ResourceManager - completed cleanup of file cache");
                    return callback();
                });
            }
            var dir = ResourceManager.cleanerDirectoryList.pop();
            return dir.listFiles(function (files) {
                try {
                    for (var ndx = 0; ndx < files.length; ndx++) {
                        var file = files[ndx];
                        if (file.isDirectory) {
                            _klogger.debug(`ResourceManager.cleanerDirectoryList.push(${file.fullPath})`);
                            ResourceManager.cleanerDirectoryList.push(file);
                        }
                        else {
                            _klogger.debug(`ResourceManager.cleanerFilesList.push(${file.fullPath})`);
                            ResourceManager.cleanerFilesList.push(file);
                        }
                    }
                    ResourceManager.cacheCleanDirectory(wipeYourCache, callback);
                }
                catch (err) {
                    _klogger.error(err);
                    return callback();
                }
            }, function (err) {
                _klogger.error(err);
                return callback();
            });
        }
        catch (err) {
            _klogger.error(err);
            return callback();
        }
    };
    ResourceManager.cacheDeleteFilesList = function (wipeYourCache, callback) {
        try {
            if (!ResourceManager.cleanerFilesList || ResourceManager.cleanerFilesList.length < 1) {
                return callback();
            }
            var file_1 = ResourceManager.cleanerFilesList.pop();
            var path = file_1.fullPath.substring(baseDirectory.length);
            if (wipeYourCache || (ResourceManager.resourceKeeper.indexOf(path) < 0 && !file_1.fullPath.startsWith('downloads/prt/'))) {
                {
                    return ResourceManager.deleteDownloadedFile(file_1.fullPath, function () {
                        _klogger.debug("##### ResourceManager ----------------------------------------------------------------------- DELETE " + file_1.fullPath);
                        return ResourceManager.cacheDeleteFilesList(wipeYourCache, callback);
                    });
                }
            }
            return setTimeout(function () {
                return ResourceManager.cacheDeleteFilesList(wipeYourCache, callback);
            }, 0);
        }
        catch (err) {
            _klogger.error(err);
            return callback();
        }
    };
    ResourceManager.cacheCleaner = function (wipeYourCache, callback) {
        try {
            if (!callback) {
                callback = function () { };
            }
            ResourceManager.cleanerDirectoryList = [];
            ResourceManager.cleanerFilesList = [];
            clearTimeout(ResourceManager.cacheCleanTimer);
            // currently downloading?
            if (wipeYourCache || (ResourceManager.resourceKeeper.length > 0 && !ResourceManager.isDownloading)) {
                if (wipeYourCache) {
                    ResourceManager.resourceKeeper = [];
                }
                _klogger.info('##### ResourceManager - cache clean up starting NOW');
                ResourceManager.cacheDeleteCutoff = moment().add(-cacheRetainDays, 'days');
                return tizen.filesystem.resolve(baseDirectory, function (downloads) {
                    ResourceManager.cleanerDirectoryList.push(downloads);
                    ResourceManager.cacheCleanDirectory(wipeYourCache, callback);
                }, function (err) {
                    _klogger.error(err);
                    return callback();
                });
            }
            else {
                ResourceManager.startCacheCleaner();
                return callback();
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.startCacheCleaner = function () {
        if (ResourceManager.countFilesDownloaded < 1) {
            _klogger.debug("##### ResourceManager - no files downloaded, skipping cache clean up pass.");
        }
        else {
            _klogger.debug("##### ResourceManager - starting wait timer for cache clean up");
            clearTimeout(ResourceManager.cacheCleanTimer);
            ResourceManager.cacheCleanTimer = setTimeout(ResourceManager.cacheCleaner, 30 * 1000);
        }
    };
    ResourceManager.downloadComplete = function () {
        try {
            // if the company splash file exists update localStorage of launch page can use it.
            if (ResourceManager.playerSplashFullPath && ResourceManager.FullDownloadPath) {
                ResourceManager.playerSplashFullPath = SourcePatcher.fixPath(ResourceManager.playerSplashFullPath);
                tizen.filesystem.resolve(ResourceManager.playerSplashFullPath, function (splashFile) {
                    if (splashFile.isFile) {
                        localStorage.setItem('KorbytPlayerSplashPath', ResourceManager.playerSplashFullPath);
                    }
                }, function (err) {
                    if (err.code === 8) {
                        _klogger.info("player splash graphic is not installed");
                    }
                    else {
                        _klogger.error(err);
                    }
                });
            }
            localStorage.removeItem('KorbytDownloadProgress');
            if (!ResourceManager.isDownloading) {
                _klogger.error("ResourceManager.downloadComplete() - ResourceManager.isDownloading === " + ResourceManager.isDownloading);
            }
            ResourceManager.isDownloading = false;
            ResourceManager.finishTime = new Date();
            var downloadTime = ResourceManager.finishTime.getTime() - ResourceManager.startTime.getTime();
            _klogger.info("##### ResourceManager - DOWNLOAD COMPLETED, downloaded " + ResourceManager.countFilesDownloaded + " of " + ResourceManager.resourceKeeper.length + " files in " + downloadTime / 1000 + " seconds");
            if (ResourceManager.cleanCacheProcess) {
                ResourceManager.startCacheCleaner();
            }
            if (ResourceManager.new_etags) {
                return ResourceManager.deleteDownloadedFile('downloads/etags.json', function () {
                    tizen.filesystem.resolve('downloads', function (dir) {
                        var etags_file = dir.createFile("etags.json");
                        if (etags_file != null) {
                            return etags_file.openStream('rw', function (fs) {
                                try {
                                    fs.write(JSON.stringify(ResourceManager.new_etags));
                                    fs.close();
                                    if (ResourceManager.finalCallback) {
                                        ResourceManager.finalCallback(null);
                                        ResourceManager.finalCallback = null;
                                    }
                                    else {
                                        _klogger.warn('ResourceManager.downloadComplete() - no final callback method found.');
                                    }
                                }
                                catch (err) {
                                    _klogger.error('ResourceManager.downloadComplete().etags_file.openStream() Exception - ' + err);
                                }
                            }, null, 'UTF-8');
                        }
                    }, function (err) {
                        _klogger.error('Failed resolve directory "downloads": ' + err);
                        if (ResourceManager.finalCallback) {
                            ResourceManager.finalCallback(null);
                            ResourceManager.finalCallback = null;
                        }
                        else {
                            _klogger.warn('ResourceManager.downloadComplete() - no final callback method found.');
                        }
                    });
                });
            }
            if (ResourceManager.finalCallback) {
                ResourceManager.finalCallback(null);
                ResourceManager.finalCallback = null;
            }
            else {
                _klogger.warn('ResourceManager.downloadComplete() - no final callback method found.');
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.queueResources = function (resources) {
        if (Array.isArray(resources) && resources.length > 0) {
            resources.forEach(function(resource) {
                if (typeof resource === 'string' && resource.startsWith('/ml/') && !ResourceManager.dlPending.includes(resource)) {
                    tizen.filesystem.resolve(resource, function(file) {
                        if (file.isFile) {
                            return;
                        }
                    }, function(err) {
                        ResourceManager.dlPending.push(resource);
                        if (!ResourceManager.resourceKeeper.includes(resource)) {
                            ResourceManager.resourceKeeper.push(resource);
                        }
                    });
                }
            });
            if (!ResourceManager.isDownloading) {
                ResourceManager.downloadNext();
            }
        }
    };
    ResourceManager.dlPending = [];
    ResourceManager.isDownloading = false;
    return ResourceManager;
}());
