/// <xreference path="../../../Dev/src/moment.d.ts"/>
/// <xreference path="../../../Dev/src/jquery.d.ts" />
/// <xreference path="../../../Dev/src/enums.ts" />
/// <xreference path="../../../Dev/src/interfaces.ts" />
/// <reference path="../../../Dev/src/utility.ts" />
/// <reference path="../../../Dev/src/logger.ts" />
/// <reference path="../../../Dev/src/blackoutschedule.ts" /> 
/// <reference path="../../../Dev/src/VisibleLogger.ts" /> 
/// <reference path="semaphore.js" />
/// <reference path="./qrcode.js" />
/// <reference path="./qrPanel.ts" />
/// <reference path="./constants.js" />
/// <xreference path="./logger.js"/>
/// <reference path="pal.Tizen.ts"/>
/// <reference path="modernizr-custom.js"/>
var _klogger = new Logger({ depth: 256, level: iLogSeverity.Warn });
var defaultCheckinInterval = 60 * 60;
;
;
var main = /** @class */ (function () {
    function main() {
    }
    main.init = function () {
        _klogger.setLogLevel(iLogSeverity.Info);
        _klogger.info('========== String.prototype.startsWith');
        if (!String.prototype.startsWith) {
            String.prototype.startsWith = function (search, pos) {
                return this.substr(!pos || pos < 0 ? 0 : +pos, search.length) === search;
            };
        }
        _klogger.info('========== String.prototype.endsWith');
        if (!String.prototype.endsWith) {
            String.prototype.endsWith = function (search, this_len) {
                if (this_len === undefined || this_len > this.length) {
                    this_len = this.length;
                }
                return this.substring(this_len - search.length, this_len) === search;
            };
        }
        if (typeof Object.assign != 'function') {
            _klogger.info('========== polyfill: Object.assign');
            // Must be writable: true, enumerable: false, configurable: true
            Object.defineProperty(Object, "assign", {
                value: function assign(target, varArgs) {
                    'use strict';
                    if (target == null) { // TypeError if undefined or null
                        throw new TypeError('Cannot convert undefined or null to object');
                    }
                    var to = Object(target);
                    for (var index = 1; index < arguments.length; index++) {
                        var nextSource = arguments[index];
                        if (nextSource != null) { // Skip over if undefined or null
                            for (var nextKey in nextSource) {
                                // Avoid bugs when hasOwnProperty is shadowed
                                if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
                                    to[nextKey] = nextSource[nextKey];
                                }
                            }
                        }
                    }
                    return to;
                },
                writable: true,
                configurable: true
            });
        }
        _klogger.info('========== main.init called');
        main.pal = getKorbytPlayerPal();
        main.cachingPlayerEnabled = false;
        main.isFirstCheckin = true;
        main.firstTimeCheckIn = true;
        main.heartbeatInterval = 60;
        main.checkinInterval = defaultCheckinInterval;
        main.nextCheckin = Date.now();
        _klogger.info('========== main.keyboardHook called');
        main.keyboardHook();
        _klogger.info('========== main.init Create File Manager Object');
        try {
            ResourceManager.initializer();
        }
        catch (err) {
            _klogger.error(err);
        }
        main.pal.enableDebugging();
        main.pal.enableSystemEvents();
        main.pal.initialize(function () {
            _klogger.debug('========== in init code');
            window.addEventListener('message', main.windowMessageListener);
            if (main.pal.player_name.trim() !== '' && (main.pal.wiredMac != '' || main.pal.wirelessMac != '')) {
                if (main.pal.provisioning && main.pal.provisioning.provisioned) {
                    return main.pal.checkProvisioning(main.pal.provisioning.webService, function () {
                        try {
                            main.pal.saveSettings(function () { });
                            _klogger.debug("========== checkProvisioning returned: " + JSON.stringify(main.pal.provisioning));
                            if (main.pal.provisioning && main.pal.provisioning.provisioned) {
                                if (main.pal.provisioning.tenant_token) {
                                    main.pal.provisioning.tenant_number = Utility.decodeTenant(main.pal.provisioning.tenant_token);
                                }
                                return main.pal.IsNetworkAvailable(function (online) {
                                    if (online) {
                                        // if network is available then always start in non-caching mode.
                                        _klogger.info("STARTING PLAYER IN NON-CACHING MODE (online:" + online + ")");
                                        main.cachingPlayerEnabled = false;
                                        main.StartPlayerRunTime();
                                    }
                                    else if (main.pal.CheckInResults && main.pal.CheckInResults.instructions) {
                                        // if network is not available and we have saved instructions then start in caching mode.
                                        _klogger.info("STARTING PLAYER IN CACHING MODE (online:" + online + ",CheckInResults:" + !!main.pal.CheckInResults + ",instructions:" + !!main.pal.CheckInResults.instructions + ")");
                                        main.cachingPlayerEnabled = true;
                                        main.StartPlayerRunTime();
                                    }
                                    else {
                                        // otherwise punt
                                        _klogger.info("NOT STARTING PLAYER --- WAITING FOR NETWORKING (online:" + online + ",CheckInResults:" + !!main.pal.CheckInResults + ",instructions:" + !!main.pal.CheckInResults.instructions + ")");
                                        main.openWindow('waiting4Server.html');
                                    }
                                    // start filling the cache with the player run-time.
                                    return ResourceManager.start(main.pal.provisioning.tenant_number, !main.pal.CheckInResults, function () {
                                        try {
                                            return main.performCheckin(function () {
                                                main.StartPlayerRunTime();
                                            });
                                        }
                                        catch (err) {
                                            _klogger.error(err);
                                        }
                                    });
                                });
                            }
                            else {
                                return main.waitForProvisioning(false);
                            }
                        }
                        catch (err) {
                            _klogger.error(err);
                        }
                    });
                }
                else {
                    return main.waitForProvisioning(false);
                }
            }
            else {
                main.pal.forceSettings = false;
                return main.waitForProvisioning(false);
            }
        });
    };
    main.StartPlayerRunTime = function () {
        try {
            var p = {
                checkin: main.pal.checkInData,
                provisioning: main.pal.provisioning
            };
            var parms = Utility.encodeURIObject(p);
            var url = main.pal.playerLoadingUrl(parms, main.cachingPlayerEnabled);
            main.openWindow(url);
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    main.keyboardHook = function () {
        try {
            var supportedKeys = tizen.tvinputdevice.getSupportedKeys();
            for (var kdx = 0; kdx < supportedKeys.length; kdx++) {
                main.keyCodes[supportedKeys[kdx].name] = supportedKeys[kdx].code;
            }
            tizen.tvinputdevice.registerKey('ColorF0Red');
            tizen.tvinputdevice.registerKey('ColorF1Green');
            tizen.tvinputdevice.registerKey('ColorF2Yellow');
            tizen.tvinputdevice.registerKey('ColorF3Blue');
            document.body.addEventListener('keydown', function() {
                main.keyboardHandler();
            });
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    main.keyboardHandler = function (e) {
        try {
            _klogger.debug("main.eventKeyboard(" + e.keyCode + "," + e.key + "," + e.metaKey + ");");
            switch (e.keyCode) {
                case main.keyCodes.ColorF0Red:
                    PlayerHost.ToggleIdentityPanel();
                    break;
                case main.keyCodes.ColorF1Green:
                    main.cachingPlayerEnabled = true;
                    main.StartPlayerRunTime();
                    break;
                case main.keyCodes.ColorF2Yellow:
                    main.cachingPlayerEnabled = false;
                    main.StartPlayerRunTime();
                    break;
                case main.keyCodes.ColorF3Blue:
                    main.pal.showVisibleLogger = !main.pal.showVisibleLogger;
                    VisibleLogger.show(main.pal.showVisibleLogger);
                    break;
                case 67: // 'C'
                    if (e.ctrlKey && e.altKey) {
                        main.deprovisioned('Keyboard CTRL+ALT+C');
                    }
                    return;
                case 88: // 'X'
                    if (e.ctrlKey && e.altKey) {
                        PlayerHost.pal.sendMessage('exit');
                    }
                    return;
                case 68: // 'D'
                    if (e.ctrlKey && e.altKey) {
                        main.pal.showVisibleLogger = !main.pal.showVisibleLogger;
                        VisibleLogger.show(main.pal.showVisibleLogger);
                    }
                    return;
                case 80: // 'P'
                    if (e.ctrlKey && e.altKey) {
                        PlayerHost.ToggleIdentityPanel();
                    }
                    return;
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    Object.defineProperty(main, "LastRestartReason", {
        get: function () {
            if (!main.lastRestartReason) {
                main.lastRestartReason = localStorage.getItem('KorbytLastRestartReason');
                localStorage.removeItem('KorbytLastRestartReason');
            }
            return main.lastRestartReason;
        },
        set: function (reason) {
            if (main.lastRestartReason != reason) {
                main.lastRestartReason = reason;
                localStorage.setItem('KorbytLastRestartReason', reason);
            }
        },
        enumerable: false,
        configurable: true
    });
    main.sendMessage = function (topic, obj) {
        try {
            if (!main.cachingPlayerEnabled) {
                var msg = { to: 'player' };
                msg[topic] = obj;
                var iframe = document.getElementById('playerContents');
                if (iframe) {
                    iframe.contentWindow.postMessage(JSON.stringify(msg), main.pal.provisioning.cms);
                    // _klogger.debug(`========== sendMessage(${JSON.stringify(msg)})`);
                }
                else {
                    _klogger.warn('main.sendMessage() -> document.playerContents not found.');
                }
            }
            else {
                _klogger.debug("can't postMessage(" + topic + ") to caching player");
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    main.setNextHeartbeat = function (resetCheckin) {
        clearTimeout(main.heartbeatTimer);
        if (resetCheckin) {
            main.nextCheckin = Date.now() + main.checkinInterval * 1000;
            _klogger.debug(new Date().toTimeString() + "| next check-in === " + new Date(main.nextCheckin).toTimeString());
        }
        var interval = main.heartbeatInterval * 1000;
        _klogger.debug(new Date().toTimeString() + "| setNextHeartbeat() === " + interval);
        main.heartbeatTimer = setTimeout(main.performHeartbeat, interval);
        if (resetCheckin && main.playerReady && main.pal.CheckInResults && (Object.keys(main.pal.CheckInResults).length != 0)) {
            main.sendMessage('checkInResults', main.pal.CheckInResults);
        }
    };
    main.performScreenShot = function (callback) {
        try {
            main.pal.captureScreenShot(function (img) {
                try {
                    _klogger.debug('uploading screen snap shot:');
                    var wsUrl = main.pal.provisioning.webService + "/v1/uploadscreenshot";
                    $.ajax({
                        async: true,
                        crossDomain: true,
                        url: wsUrl,
                        method: "POST",
                        headers: {
                            'Authorization': 'Bearer ' + main.bearer
                        },
                        contentType: false,
                        processData: false,
                        cache: false,
                        data: img,
                        statusCode: {
                            410: function () {
                                main.deprovisioned();
                                main.setNextHeartbeat(false);
                                if (callback)
                                    callback();
                            }
                        },
                        success: function (response) {
                            _klogger.info("screenshot success: " + JSON.stringify(response));
                            if (callback)
                                callback();
                        },
                        error: function (err) {
                            if (err.responseText) {
                                _klogger.error("screenshot failed: " + err.responseText);
                            }
                            else {
                                _klogger.error("screenshot failed: " + err);
                            }
                            if (callback)
                                callback();
                        }
                    });
                }
                catch (err) {
                    _klogger.error(err);
                    if (callback)
                        callback();
                }
            });
        }
        catch (err) {
            _klogger.error(err);
            if (callback)
                callback();
        }
    };
    main.performCheckin = function (callback) {
        return tizen.systeminfo.getPropertyValue('NETWORK', function (networkInfo) {
            try {
                if (networkInfo.networkType === 'NONE') {
                    _klogger.warn("Skipping player check-in, network disconnected");
                    try {
                        if (callback)
                            callback();
                    }
                    catch (err) {
                        _klogger.error(err);
                    }
                    return main.setNextHeartbeat(true);
                }
                // go get the current check-in information.
                return main.pal.checkinGetInfo(function (ciData, err) {
                    try {
                        if (err) {
                            _klogger.error(err);
                            main.setNextHeartbeat(true);
                            if (callback)
                                callback();
                        }
                        else if (!ciData.macAddress || ciData.macAddress.length < 1) {
                            _klogger.error("Check-in data does not include MAC address");
                            main.setNextHeartbeat(true);
                            if (callback)
                                callback();
                        }
                        else {
                            _klogger.debug(JSON.stringify(ciData));
                            return $.ajax({
                                "async": true,
                                "crossDomain": true,
                                "url": main.pal.provisioning.webService + "/player/checkin/v3",
                                "method": "POST",
                                "headers": {
                                    "content-type": "application/json",
                                    "cache-control": "no-cache",
                                },
                                "processData": false,
                                "data": JSON.stringify(ciData),
                                statusCode: {
                                    410: function () {
                                        try {
                                            main.deprovisioned();
                                            main.setNextHeartbeat(true);
                                        }
                                        catch (err) {
                                            _klogger.error(err);
                                        }
                                        if (callback)
                                            callback();
                                    }
                                },
                                success: function (cir) {
                                    try {
                                        if (cir.instructions && cir.instructions.resources) {
                                            // main.pal.CheckInResults = cir;
                                            PlayerHost.blackoutschedule = new BlackoutSchedule(cir.instructions.metadata);
                                            main.pal.playerID = cir.playerID;
                                            main.bearer = cir.bearer;
                                            _klogger.log("---------- Check-in response: " + JSON.stringify(cir));
                                            try {
                                                if (cir.instructions && cir.instructions.metadata && cir.instructions.metadata.properties && cir.instructions.metadata.properties.orientation) {
                                                    main.pal.setOrientation(parseInt(cir.instructions.metadata.properties.orientation));
                                                }
                                            }
                                            catch (err) {
                                                _klogger.error(err);
                                            }
                                            try {
                                                if (cir.instructions && cir.instructions.metadata && cir.instructions.metadata.properties && cir.instructions.metadata.properties.visualConsole) {
                                                    main.pal.showVisibleLogger = parseInt(cir.instructions.metadata.properties.visualConsole) === 1;
                                                    _klogger.info("instructions.metatdata.properties.visualConsole === " + main.pal.showVisibleLogger);
                                                    VisibleLogger.show(main.pal.showVisibleLogger);
                                                }
                                            }
                                            catch (err) {
                                                _klogger.error(err);
                                            }
                                            // try {
                                            //     if (cir.instructions.metadata && cir.instructions.metadata.screen_control && cir.instructions.metadata.screen_control.schedule) {
                                            //         var screenControlSchedules = cir.instructions.metadata.screen_control.schedule;
                                            //         for (var _i = 0; _i < screenControlSchedules.length; _i++) {
                                            //             var command = screenControlSchedules[_i].cmd;
                                            //             switch (command) {
                                            //                 case 'volume':
                                            //                     if(screenControlSchedules[_i].value){
                                            //                         _klogger.info("---Sound Level Request---" + screenControlSchedules[_i].value);
                                            //                         main.pal.setSoundVolume(parseInt(screenControlSchedules[_i].value));
                                            //                     }
                                            //                     break;
                                            //                 default:
                                            //                     _klogger.info("---Screen Control Not supported by SOC---")
                                            //                     break;
                                            //             }
                                            //         }
                                            //     }
                                            // }
                                            // catch (err) {
                                            //     _klogger.error("---Screen Control Error---" + err);
                                            // }
                                            if (main.pal.provisioning.tenant_token) {
                                                main.pal.provisioning.tenant_number = Utility.decodeTenant(main.pal.provisioning.tenant_token);
                                            }
                                            // if we are not currently playing cached content, notify the PRT now.
                                            if (!main.cachingPlayerEnabled) {
                                                main.sendMessage('checkInResults', cir);
                                            }
                                            if (!cir.instructions || !main.pal.CheckInResults || !main.pal.CheckInResults.instructions || main.makeComparable(cir.instructions) != main.makeComparable(main.pal.CheckInResults.instructions)) {
                                                main.pal.CheckInResults = cir;
                                                main.cachingPlayerEnabled = false;
                                                main.StartPlayerRunTime();
                                                _klogger.log('START STREAMING MODE');
                                            }
                                            var resources = [];
                                            for (var _i = 0, _a = cir.instructions.resources; _i < _a.length; _i++) {
                                                var resource = _a[_i];
                                                resources.push(resource.fn);
                                            }
                                            ResourceManager.getResources(true, resources, !main.pal.CheckInResults, function () {
                                                _klogger.info('Resource manager download completed');
                                                // if (cir.instructions.playlists_master) {
                                                //     const masterPlayList = cir.instructions.playlists_master;
                                                //     for (const key in cir.instructions.playlists_master) {
                                                //         if (masterPlayList.hasOwnProperty(key)) {
                                                //             const playList = cir.instructions.playlists_master[key];
                                                //             for (let idx = 0; idx < playList.items.length; idx++) {
                                                //                 const item = playList.items[idx];
                                                //                 if (item && item.uri) {
                                                //                     item.uri = SourcePatcher.fixPath(item.uri);
                                                //                 }
                                                //             }
                                                //         }
                                                //     }
                                                // }
                                                main.pal.CheckInResults = cir;
                                                return main.pal.saveSettings(function () {
                                                    if (!main.cachingPlayerEnabled) {
                                                        console.log('START CACHED MODE');
                                                        main.cachingPlayerEnabled = true;
                                                        main.StartPlayerRunTime();
                                                    }
                                                    if (main.isFirstCheckin) {
                                                        setTimeout(main.performScreenShot, 30 * 1000);
                                                        main.isFirstCheckin = false;
                                                    }
                                                    //main.setNextHeartbeat(true);
                                                });
                                            });
                                        }
                                        else {
                                            if (main.firstTimeCheckIn) {
                                                main.firstTimeCheckIn = false;
                                                return setTimeout(function () {
                                                    // First Checkin, never returns instructions ... repeat checkin
                                                    main.performCheckin(callback); // Run again immediately
                                                }, 2000);
                                            }
                                            return callback();
                                        }
                                    }
                                    catch (err) {
                                        _klogger.error(err);
                                    }
                                    main.setNextHeartbeat(true);
                                    if (callback) {
                                        return callback();
                                    }
                                },
                                error: function (err) {
                                    main.setNextHeartbeat(true);
                                    _klogger.error("Check-in failed: " + JSON.stringify(err));
                                    if (callback)
                                        callback();
                                }
                            });
                        }
                    }
                    catch (err) {
                        main.setNextHeartbeat(true);
                        _klogger.error(err);
                        if (callback)
                            callback();
                    }
                });
            }
            catch (err) {
                _klogger.error(err);
            }
        });
    };
    main.makeComparable = function (_ins) {
        if (!_ins) {
            _klogger.error('main.makeComparable() - invalid parameter');
            return undefined;
        }
        if (typeof _ins === 'object') {
            var ins = JSON.parse(JSON.stringify(_ins)); // Clone it first
            if (ins.createtime) {
                delete ins.createtime;
            }
            if (ins.resources) {
                delete ins.resources;
            }
            if (ins.host) {
                delete ins.host; // WS host may be different
            }
            if (typeof ins.offline !== 'undefined') {
                delete ins.offline; // Don't care if these started out offline or not
            }
            if (typeof ins.current_item !== 'undefined') {
                delete ins.current_item;
            }
            for (var id in ins.playlists_master) {
                var pl = ins.playlists_master[id];
                if (pl.lastupdate) {
                    delete pl.lastupdate;
                }
                if (pl.updated_dt) {
                    delete pl.updated_dt;
                }
                if (pl.name) {
                    delete pl.name;
                }
                if (pl.playtimes) {
                    // These change on every save
                    for (var t = 0; t < pl.playtimes.length; t++) {
                        if (typeof pl.playtimes[t].id != "undefined") {
                            delete pl.playtimes[t].id;
                        }
                    }
                }
            }
            return JSON.stringify(ins);
        }
    };
    main.performHeartbeat = function (checkinNow) {
        try {
            if (!main.playerReady) {
                return _klogger.warn('no heartbeat --- PLAYER NOT READY');
            }
            // are we currently provisioned?
            if (main.pal.provisioning.provisioned) {
                // is it time to do a check-in?
                if (checkinNow || main.pal.playerID < 1 || !main.nextCheckin || main.nextCheckin < Date.now()) {
                    try {
                        if (checkinNow || !PlayerHost.blackoutschedule || !PlayerHost.blackoutschedule.inBlackout()) {
                            _klogger.debug("Heartbeat: playerID=" + main.pal.playerID + " calling performChecking()");
                            return main.performCheckin(function () {
                                return main.setNextHeartbeat(true);
                            });
                        }
                    }
                    catch (e) {
                        _klogger.error(e);
                    }
                }
                return main.pal.getStorageLoad(function (storageLoad) {
                    return main.pal.getMemoryLoad(function (memLoad) {
                        return main.pal.getCpuLoad(function (cpuLoad) {
                            _klogger.info("Performing heartbeat: PID=" + main.pal.playerID);
                            return $.ajax({
                                url: main.pal.provisioning.webService + "/v1/heartbeat?pid=" + main.pal.playerID + "&start=" + main.startTime.toISOString() + "&cpu=" + cpuLoad + "&storage=" + storageLoad + "&memory=" + memLoad + "&reason=" + main.LastRestartReason,
                                headers: { 'Authorization': 'Bearer ' + main.bearer },
                                statusCode: {
                                    410: function () {
                                        main.deprovisioned();
                                        return main.setNextHeartbeat(false);
                                    }
                                },
                                success: function (data) {
                                    main.heartbeatWork(data);
                                    return main.setNextHeartbeat(false);
                                },
                                error: function (data) {
                                    _klogger.error("Error - " + JSON.stringify(data));
                                    return main.setNextHeartbeat(false);
                                },
                                type: "GET",
                                dataType: "json"
                            });
                        });
                    });
                });
            }
            else {
                _klogger.log("No heartbeat performed - not provisioned.");
                return main.setNextHeartbeat(false);
            }
        }
        catch (err) {
            _klogger.error(err);
            _klogger.error(err);
            main.setNextHeartbeat(false);
        }
    };
    main.clearCache = function (callback) {
        if (!main.isClearingCache) {
            main.isClearingCache = true;
            main.pal.FactoryReset();
            ResourceManager.FullDownloadPath = undefined;
            ResourceManager.PlayerSplashFullPath = undefined;
            main.firstTimeCheckIn = true;
            main.bearer = undefined;
            main.cachingPlayerEnabled = false;
            localStorage.clear();
            main.openWindow('provisioning.html');
            return ResourceManager.cacheCleaner(true, function () {
                _klogger.info("Cache cleaner completed.");
                return callback();
            });
        }
    };
    main.heartbeatWork = function (json) {
        var queue = json.actions;
        var result = '';
        _klogger.log(json);
        main.heartbeatInterval = (json.nextbeat || 60);
        main.checkinInterval = (json.nextCheckin || defaultCheckinInterval);
        var nextWorkItem = function () {
            if (queue.length) {
                _klogger.info(queue.join('|'));
                if (main.findAndRemove(queue, 'checkin')) {
                    if (PlayerHost.blackoutschedule && PlayerHost.blackoutschedule.inBlackout()) {
                        _klogger.info('Ignoring checkin, player is currently in a blackout time period');
                        return nextWorkItem();
                    }
                    main.performHeartbeat(true);
                    return nextWorkItem();
                }
                else if (main.findAndRemove(queue, 'urgent')) {
                    main.performHeartbeat(true);
                    return nextWorkItem();
                }
                else if (main.findAndRemove(queue, 'screenshot')) {
                    return main.performScreenShot(nextWorkItem);
                }
                else if (main.findAndRemove(queue, 'playerlog')) {
                    return main.uploadPlayerLog(nextWorkItem);
                }
                else if (result = main.findAndRemove(queue, 'sleepTimes##')) {
                    return main.pal.setSleepSchedule(nextWorkItem, result);
                }
                else if (result = main.findAndRemove(queue, 'rename##')) {
                    main.pal.player_name = result;
                    main.pal.checkInData.player_name = result;
                    return main.pal.saveSettings(nextWorkItem);
                }
                else if (result = main.findAndRemove(queue, 'upgrade##')) {
                    return main.pal.upgradePlayer(nextWorkItem, result);
                }
                else if (result = main.findAndRemove(queue, 'volume##')) {
                    try {
                        var level = parseInt(result);
                        if (level >= 0 && level <= 100) {
                            main.pal.checkInData.volume = "" + level;
                            main.pal.setSoundVolume(level);
                            return main.pal.saveSettings(nextWorkItem);
                        }
                        _klogger.error("Sound volume out of range " + level + "(" + result + ")");
                        return nextWorkItem();
                    }
                    catch (err) {
                        _klogger.error(err);
                        return nextWorkItem();
                    }
                }
                else if (result = main.findAndRemove(queue, 'properties##')) {
                    var show = (typeof result === 'string') && (result.toLowerCase() === 'true');
                    PlayerHost.ShowIdentityPanel(show);
                    return nextWorkItem();
                }
                else if (result = main.findAndRemove(queue, 'clear-cache')) {
                    main.deprovisioned('cache cleared (verb)');
                    return nextWorkItem();
                }
                else if (result = main.findAndRemove(queue, 'show-QR')) {
                    main.qrCodeShowing = true;
                    QRPanel.show({ player_name: main.pal.player_name, player_id: main.pal.playerID, tenant_id: main.pal.provisioning.tenant_number });
                    return nextWorkItem();
                }
                else if (result = main.findAndRemove(queue, 'hide-QR')) {
                    main.qrCodeShowing = false;
                    QRPanel.hide();
                    return nextWorkItem();
                }
                else if (main.findAndRemove(queue, 'reboot')) {
                    main.pal.reboot('reboot verb');
                    return nextWorkItem();
                }else if(main.findAndRemove(queue, 'restart')){
                    try{
                        var appId = tizen.application.getCurrentApplication().appInfo.id;
                        var appControl = new tizen.ApplicationControl("http://tizen.org/appcontrol/operation/restart");
                        tizen.application.launchAppControl(appControl,appId);
                    }catch(e){
                        _klogger.error("Unable to restart app" + e.message);
                    }
                    return nextWorkItem();
                }
            }
        };
        nextWorkItem(); // kick it off
    };
    main.findAndRemove = function (arr, prefix) {
        for (var i = 0; i < arr.length; i++) {
            if (arr[i].indexOf(prefix) === 0) {
                var result = 'true';
                if (prefix.lastIndexOf('##') === (prefix.length - 2)) {
                    result = arr[i].replace(prefix, '').replace('##', '');
                }
                arr.splice(i, 1);
                return result;
            }
        }
        return undefined;
    };
    main.notSupported = function (feature, callback) {
        _klogger.warn(feature + " not supported");
        callback();
    };
    main.windowMessageListener = function (event) {
        var msg = { to: null };
        try {
            var d = event.data;
            var s = d.indexOf('{');
            var e = d.lastIndexOf('}');
            if (s === 0 && e === d.length - 1) {
                msg = JSON.parse(event.data);
            }
        }
        catch (err) {
            _klogger.error("Unable to parse message: " + event.data + "\n" + err.stack);
        }
        if (msg.to) {
            switch (msg.to) {
                case 'agent':
                case 'host':
                    if (msg.message) {
                        switch (msg.message) {
                            case 'playerReady':
                                console.info('Got player ready message');
                                main.playerReady = true;
                                return main.setNextHeartbeat(true);
                            case 'restart':
                                console.info('Got player restart');
                                return main.pal.restart('restart message');
                            case 'reboot':
                                console.info('Got player reboot');
                                return main.pal.reboot('reboot message');
                            case 'exit':
                                _klogger.warn("don't know how to exit yet: " + JSON.stringify(msg));
                                return;
                            case 'deprovision':
                                _klogger.warn("Player de-provisioned: " + JSON.stringify(msg));
                                return main.deprovisioned();
                            case 'cacheclear':
                                _klogger.warn("Player de-provisioned: " + JSON.stringify(msg));
                                return main.deprovisioned('Cache Cleared (keyboard)');
                            case 'fetchResources':
                                _klogger.info("Fetch data from subscriptions: " + JSON.stringify(msg));
                                return main.mediaLibraryFetchResources(msg);
                            default:
                                _klogger.warn("Message ignored: " + JSON.stringify(msg));
                                return console.log("Message ignored: " + JSON.stringify(msg));
                        }
                    }
                    if (msg.logger) {
                        return _klogger.logged(msg.logger);
                    }
                    break;
                case 'player':
                    // Korbyt.PlayerEngine.postMessage(msg);
                    break;
                default:
                    _klogger.info("Message to " + msg.to + " ignored: " + event.data);
                    break;
            }
        }
    };
    main.uploadPlayerLog = function (callback) {
        try {
            if (main.pal.provisioning.provisioned) {
                var logs = _klogger.stringify();
                var wsUrl = main.pal.provisioning.webService + "/v1/playerlog";
                // const wsUrl = `http://rrogers:8080/ws/v1/playerlog`;
                $.ajax({
                    async: true,
                    crossDomain: true,
                    url: wsUrl,
                    method: "POST",
                    headers: {
                        Authorization: 'Bearer ' + main.bearer
                    },
                    processData: false,
                    contentType: 'text/plain',
                    cache: false,
                    data: logs,
                    statusCode: {
                        410: function () {
                            main.deprovisioned();
                            return main.setNextHeartbeat(false);
                        }
                    },
                    success: function (response) {
                        try {
                            console.info("playerlog upload complete.");
                        }
                        catch (err) {
                            console.error(err);
                            _klogger.error(err);
                        }
                        return callback();
                    },
                    error: function (err) {
                        console.error(err);
                        _klogger.error("playerlog upload failed: " + JSON.stringify(err));
                        return callback();
                    }
                });
            }
        }
        catch (err) {
            console.error(err);
            _klogger.error(err);
            return callback();
        }
    };
    main.openWindow = function (path) {
        try {
            _klogger.info("========== ========== ========== ========== openWindow: " + path);
            var inner = "<iframe src=\"" + path + "\" width=\"100%\" height=\"100%\" id=\"playerContents\" target=\"playerContents\" name=\"playerContents\"></iframe>";
            $('#playerhost').html(inner);
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    main.waitForProvisioning = function (waitingOnServer) {
        if (waitingOnServer) {
            main.openWindow('waiting4Server.html');
        }
        else {
            main.openWindow('provisioning.html');
        }
        var candidates = [];
        if (main.pal.wsUrl && (main.pal.wsUrl != '')) {
            candidates.push(main.pal.wsUrl);
        }
        else {
            candidates = Constants.provisioningCandidates;
        }
        return Utility.waitForProvisioning(main.pal.provisioningToken, candidates, function (p) {
            try {
                if (p && p.provisioned) {
                    if (p.tenant_type === 'hybrid') {
                        return Utility.checkProvisioning(main.pal.provisioningToken, p.webService, function (p) {
                            if (p && p.provisioned) {
                                main.pal.provisioning = p;
                                return main.pal.saveSettings(function () {
                                    main.pal.restart('provisioned');
                                });
                            }
                            main.waitForProvisioning(true);
                        });
                    }
                    else {
                        main.pal.provisioning = p;
                        main.pal.wsUrl = p.webService;
                        return main.pal.saveSettings(function () {
                            main.pal.restart('provisioned');
                        });
                    }
                }
            }
            catch (err) {
                console.error(err);
                _klogger.error(err);
            }
            main.waitForProvisioning(false);
        });
    };
    main.deprovisioned = function (reason) {
        try {
            if (typeof reason === 'undefined') {
                reason = 'deprovisioned';
            }
            main.pal.CheckInResults = {};
            ResourceManager.PlayerSplashFullPath = null;
            main.pal.provisioning = { provisioned: false };
            localStorage.clear();
            return main.pal.saveSettings(function () {
                return main.clearCache(function () {
                    _klogger.warn("player " + main.pal.playerID + " - " + reason);
                    return main.pal.restart(reason);
                });
            });
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    main.mediaLibraryFetchResources = function (msg) {
        try {
            if (msg && msg.data) {
                var subscriptionResources = msg.data;
                ResourceManager.queueResources(subscriptionResources);
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    main.startTime = new Date();
    main.keyCodes = {};
    main.isClearingCache = false;
    return main;
}());
