var SCREEN_SAVER_OFF = 0;
var SCREEN_SAVER_ON = 1;
var KorbytSettingsFilename = "Korbyt_Settings.json";
var b2bcontrol = null;
var _globablPal = null;
var prtVersion = "v3.12";
function getKorbytPlayerPal() {
    if (!_globablPal) {
        _globablPal = new TizenPal();
    }
    return _globablPal;
}
var TizenPal = /** @class */ (function () {
    function TizenPal() {
        this.showVisibleLogger = false;
        this.orientation = 0;
        this.screenSaverOff = false;
        this.isInitialized = false;
        this.screenSaverOff = false;
    }
    Object.defineProperty(TizenPal.prototype, "CheckInResults", {
        get: function () {
            try {
                if (!this.__checkInResults) {
                    this.__checkInResults = localStorage.getItem('instructions');
                    if (typeof this.__checkInResults === 'string') {
                        this.__checkInResults = JSON.parse(this.__checkInResults);
                    }
                }
            }
            catch (err) {
                console.error(err);
            }
            return this.__checkInResults;
        },
        set: function (cir) {
            if (cir) {
                this.__checkInResults = cir;
                _klogger.info("localStorage.setItem ---  instructions === " + JSON.stringify(this.__checkInResults.instructions));
                localStorage.setItem('instructions', JSON.stringify(this.__checkInResults));
            }
            else {
                localStorage.removeItem('instructions');
                _klogger.warn("CheckInResults === null --- cleared");
            }
        },
        enumerable: false,
        configurable: true
    });
    TizenPal.prototype.macAddress = function (ni) {
        console.log("---------- TizenPal.macAddress");
        var macAddress = [];
        for (var _interface in ni) {
            var map = [];
            for (var i = 0; i < ni[_interface].length; i++) {
                if (ni[_interface][i].address) {
                    var mac = ni[_interface][i].mac;
                    if (mac && mac !== "" && !map[mac]) {
                        macAddress.push({ 'interface': _interface, 'address': mac });
                        map[mac] = true;
                    }
                }
            }
        }
        return macAddress;
    };
    Object.defineProperty(TizenPal.prototype, "provisioningToken", {
        get: function () {
            if (!TizenPal.__provisioningToken) {
                console.log("---------- TizenPal.provisioningToken");
                if (this.wiredMac) {
                    TizenPal.__provisioningToken = Utility.encodeMac(this.wiredMac);
                }
            }
            return TizenPal.__provisioningToken;
        },
        enumerable: false,
        configurable: true
    });
    TizenPal.prototype.provisioningDetails = function (callback) {
        console.log("---------- TizenPal.provisioningDetails");
        var self = this;
        return self.checkinGetInfo(function (ci, err) {
            var details = [];
            try {
                var gotMac = false;
                details.push({ n: 'Model', v: ci.model });
                details.push({ n: 'Firmware', v: self.firmware });
                details.push({ n: 'Name', v: ci.player_name });
                details.push({ n: 'Software Version', v: ci.version });
                details.push({ n: 'Serial Number', v: ci.serialNumber });
                for (var i_1 in ci.networkInterfaces) {
                    var nia = ci.networkInterfaces[i_1];
                    for (var j in nia) {
                        var ni = nia[j];
                        if (ni.address && ni.address != '') {
                            if (ni.mac && ni.mac != '') {
                                details.push({ n: 'MAC Address', v: ni.mac.replace(/:/g, '') });
                            }
                            if (!gotMac) {
                                gotMac = true;
                                details.push({ n: 'IP Address', v: ni.address });
                            }
                        }
                    }
                }
            }
            catch (err) {
                var errMsg = "provisioningDetails() failed: " + JSON.stringify(err);
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            callback(details);
        });
    };
    TizenPal.prototype.checkProvisioning = function (webService, callback) {
        console.log("---------- TizenPal.checkProvisioning " + webService);
        var self = this;
        return tizen.systeminfo.getPropertyValue('NETWORK', function (networkInfo) {
            try {
                if (networkInfo.networkType === 'NONE') {
                    _klogger.warn("checkProvisioning() - network disconnected");
                    return callback();
                }
                console.log("TizenPal.checkProvisioning(" + webService + ")");
                var url = webService + ("/v1/check?pt=" + self.provisioningToken);
                console.info(url);
                try {
                    return $.ajax({
                        url: webService + ("/v1/check?pt=" + self.provisioningToken),
                        type: "GET",
                        dataType: "json",
                        success: function (data) {
                            // console.info("---------- checkProvisioning: " + JSON.stringify(data));
                            if (data.provisioned) {
                                self.provisioning = data;
                                return callback();
                            }
                            else {
                                return callback();
                            }
                        },
                        error: function (xhr, ajaxOptions, thrownError) {
                            console.error("pal.checkProvisioning() Error - " + thrownError || ajaxOptions);
                            _klogger.error(thrownError);
                            return callback();
                        }
                    });
                }
                catch (err) {
                    _klogger.error(err);
                    return callback();
                }
            }
            catch (err) {
                _klogger.error(err);
                return callback();
            }
        });
    };
    TizenPal.prototype.parseSettingsFile = function (settings) {
        console.log("---------- TizenPal.parseSettingsFile " + settings);
        try {
            if (!settings) {
                settings = {};
            }
            this.wsUrl = settings.wsUrl || '';
            this.wiredMac = (settings.wiredMac || '').toLowerCase();
            this.wirelessMac = (settings.wirelessMac || '').toLowerCase();
            this.player_name = settings.player_name || '';
            this.playerID = settings.playerID || 0;
            this.provisioning = settings.provisioning || { provisioned: false };
            this.serialNumber = settings.serialNumber;
            this.orientation = settings.orientation;
            if (!this.CheckInResults) {
                if (typeof settings.checkInResults === 'string') {
                    settings.checkInResults = JSON.parse(settings.checkInResults);
                }
                if (settings.checkInResults) {
                    this.CheckInResults = settings.checkInResults;
                }
            }
            ResourceManager.FullDownloadPath = settings.FullDownloadPath;
            ResourceManager.PlayerSplashFullPath = settings.PlayerSplashFullPat;
        }
        catch (err) {
            console.error(err);
        }
    };
    TizenPal.prototype.FactoryReset = function () {
        this.checkInData = undefined;
        this.playerID = undefined;
        this.wsUrl = undefined;
        this.player_name = undefined;
        this.wiredMac = undefined;
        this.wirelessMac = undefined;
        this.provisioning = { provisioned: false, webService: '' };
        this.setOrientation(0);
        this.CheckInResults = undefined;
        localStorage.removeItem('KorbytSettings');
    };
    TizenPal.prototype.loadSettings = function (callback) {
        console.log("---------- TizenPal.loadSettings");
        var self = this;
        try {
            var settingsObj = localStorage.getItem('KorbytSettings');
            try {
                settingsObj = JSON.parse(settingsObj);
            }
            catch (err) {
                console.error('################################################################################');
                console.error(err);
                console.error('################################################################################');
                console.error(settingsObj);
                console.error('################################################################################');
            }
            self.parseSettingsFile(settingsObj);
            return callback();
        }
        catch (err) {
            var errMsg = "checkProvisioning() failed: " + JSON.stringify(err);
            console.error(errMsg);
            _klogger.error(errMsg);
            self.parseSettingsFile({});
            self.saveSettings(callback);
        }
    };
    TizenPal.prototype.saveSettings = function (callback) {
        console.log("---------- TizenPal.saveSettings");
        var self = this;
        try {
            var settings = {
                wsUrl: self.wsUrl || '',
                wiredMac: self.wiredMac || '',
                wirelessMac: self.wirelessMac || '',
                player_name: self.player_name || '',
                playerID: self.playerID || 0,
                provisioning: self.provisioning || { provisioned: false, webService: '' },
                serialNumber: self.serialNumber,
                orientation: self.orientation,
                checkInResults: self.CheckInResults,
                FullDownloadPath: ResourceManager.FullDownloadPath,
                PlayerSplashFullPath: ResourceManager.PlayerSplashFullPath,
            };
            var textData = JSON.stringify(settings);
            localStorage.setItem('KorbytSettings', textData);
            return callback();
        }
        catch (err) {
            var errMsg = "saveSettings() failed: " + err;
            console.error(errMsg);
            _klogger.error(errMsg);
            return callback();
        }
    };
    TizenPal.prototype.checkinCompleted = function (callback) {
        var self = this;
        try {
            console.log("---------- TizenPal.checkinCompleted");
            if (!self.player_name || self.player_name === '') {
                var pn = "tizen-" + self.wiredMac || self.wirelessMac;
                pn = pn.replace(/:/g, '');
                self.player_name = pn;
                self.checkInData.player_name = pn;
            }
            self.checkInData.freeMemBytes = tizen.systeminfo.getAvailableMemory();
            self.checkInData.totalMemBytes = tizen.systeminfo.getTotalMemory();
            self.checkInData.freemem = Utility.convertToHumanReadable(self.checkInData.freeMemBytes);
            self.checkInData.totalmem = Utility.convertToHumanReadable(self.checkInData.totalMemBytes);
            self.checkInData.volume = tizen.tvaudiocontrol.getVolume();
            self.checkInData.upgradeFilePattern = '^[a-zA-Z0-9-_ .]+[.]wgt$';
            // update current screen orientations
            if (window.screen && window.screen.orientation && typeof (window.screen.orientation.angle) === 'number') {
                this.orientation = window.screen.orientation.angle;
            }
            self.checkInData.properties = [
                {
                    name: "orientation",
                    displayName: "Orientation",
                    tooltip: "Display orientation",
                    type: "dropdown",
                    default: "" + self.orientation,
                    values: [
                        ["0", "Landscape"],
                        ["90", "Portrait"]
                    ]
                }, {
                    name: "visualConsole",
                    displayName: "Visual Console",
                    tooltip: "Display console messages",
                    type: "dropdown",
                    default: "" + (self.showVisibleLogger ? 1 : 0),
                    values: [
                        ["0", "Disabled"],
                        ["1", "Enabled"]
                    ]
                }
            ];
            self.checkInData.capabilities = {
                screenShot: true,
                sleepSchedule: false,
                uploadLogs: true,
                remoteUpgrade: true,
                // videoFileExtensions: [".asf", ".wmv", ".divx", ".avi", ".mp4", ".m4v", ".mov", ".3gp", ".3g2", ".mkv", ".ts", ".trp", ".tp", ".mts", ".mpg", ".mpeg", ".dat", ".vob", ".webm"],
                // videoCodecs: ["H.264", "AVC", "MPEG-4", "DivX3.11", "DivX4", "DivX5", "DivX6", "XVID", , "MJpeg", "VC-1", "VP8", "VP9"],
                // audioFileExtensions: [".mp3", ".wav", ".ogg", ".wma"],
                // audioCodecs: ["Vorbis", "Opus", "AAC", "PCM", "MP3", "DTS", "LPCM", "ADPCM", "Dolby Digital", "Dolby Digital Plus", "WMA 9", "WMA", "AMR-NB", "AMR-WB"]
                verbs: [
                    "checkin",
                    "clear-cache",
                    "hide-QR",
                    // "hotkey",
                    "overlay",
                    "playerlog",
                    //"remove",
                    "rename",
                    "show-QR",
                    //"setresolution",
                    "screenshot",
                    "sleepTimes",
                    //"upgrade",
                    "urgent",
                    "volume"
                ],
                hotkeys: [
                    //"ARROWDOWN:Y:Rotate Display 180 degrees",
                    //"ARROWLEFT:Y:Rotate Display 270 degrees",
                    //"ARROWRIGHT:Y:Rotate Display 90 degrees",
                    //"ARROWUP:Y:Rotate Display 0 degrees",
                    "C:N:Clear Cache",
                    //"D:N:Toggle Debugger",
                    "I:N:Toggle Identity",
                    //"L:Y:Login",
                    //"Q:N:QR code toggle",
                    "R:Y:Reboot",
                    //"U:N:UI Debugger",
                    "X:Y:Player Exit",
                ]
            };
            try {
                if (b2bcontrol && !this.serialNumber) {
                    this.serialNumber = b2bcontrol.getSerialNumber();
                    self.checkInData.serialNumber = this.serialNumber;
                    return this.saveSettings(function () {
                        return callback(self.checkInData, null);
                    });
                }
                self.checkInData.serialNumber = this.serialNumber;
            }
            catch (e) {
                _klogger.log("[getSerialNumber] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
            }
            // self.checkInData.userAgent = navigator.userAgent;
            return callback(self.checkInData, null);
        }
        catch (err) {
            _klogger.error(err);
            return callback(self.checkInData, null);
        }
    };
    TizenPal.prototype.setOrientation = function (orient) {
        // const self = this;
        try {
            if (typeof (orient) === 'string') {
                orient = parseInt(orient);
            }
            if (orient === NaN) {
                orient = 0;
            }
            var b2bOrientVal = void 0;
            switch (orient) {
                case 270:
                case 90:
                    b2bOrientVal = 'portrait-primary';
                    break;
                case 180:
                default:
                    b2bOrientVal = 'landscape-primary';
                    break;
            }
            if (window.screen.orientation && window.screen.orientation.lock && typeof (window.screen.orientation.lock) === 'function') {
                window.screen.orientation.lock(b2bOrientVal);
            }
            else {
                // do this three times because Samsung said so.
                window.screen.lockOrientation(b2bOrientVal);
                window.screen.lockOrientation(b2bOrientVal);
                window.screen.lockOrientation(b2bOrientVal);
            }
            if (this.orientation != orient) {
                this.orientation = orient;
                main.nextCheckin = Date.now();
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    TizenPal.prototype.setSoundVolume = function (sndLevel) {
        try {
            if (sndLevel < 0)
                sndLevel = 0;
            if (sndLevel > 100)
                sndLevel = 100;
            if (sndLevel === 0) {
                tizen.tvaudiocontrol.setMute(true);
                console.log('TV audio muted');
            }
            else {
                tizen.tvaudiocontrol.setVolume(sndLevel);
                console.log("TV audio volume = " + sndLevel);
            }
        }
        catch (err) {
            console.error("setSoundVolume: " + err);
            _klogger.error(err);
        }
    };
    TizenPal.prototype.getCpuLoad = function (callback) {
        return tizen.systeminfo.getPropertyValue("CPU", function (cpu) {
            return callback(cpu.load);
        }, function (err) {
            _klogger.error(err);
            return callback(undefined);
        });
    };
    TizenPal.prototype.getMemoryLoad = function (callback) {
        return callback((tizen.systeminfo.getAvailableMemory() / tizen.systeminfo.getTotalMemory()) * 100);
    };
    TizenPal.prototype.getStorageLoad = function (callback) {
        return tizen.systeminfo.getPropertyValue("STORAGE", function (storage) {
            try {
                if (storage && storage.units && storage.units.length > 0) {
                    var unit = storage.units[0];
                    return callback((unit.availableCapacity / unit.capacity) * 100);
                }
            }
            catch (err) {
                _klogger.error(err);
            }
            return callback(undefined);
        }, function (err) {
            _klogger.error(err);
            return callback(undefined);
        });
    };
    TizenPal.prototype.checkinGetCpuLoad = function (callback) {
        var self = this;
        console.log("---------- TizenPal.checkinGetCpuLoad");
        return tizen.systeminfo.getPropertyValue("CPU", function (cpu) {
            self.checkInData.cpuUsage = cpu.load;
            return self.checkinCompleted(callback);
        }, function (err) {
            console.error("checkinGetCpuLoad: " + err);
            _klogger.error(err);
            return self.checkinCompleted(callback);
        });
    };
    ;
    TizenPal.prototype.checkinGetStorage = function (callback) {
        var self = this;
        console.log("---------- TizenPal.checkinGetStorage");
        return tizen.systeminfo.getPropertyValue("STORAGE", function (storage) {
            try {
                if (storage && storage.units && storage.units.length > 0) {
                    var unit = storage.units[0];
                    self.checkInData.freeDiskSpaceBytes = unit.availableCapacity;
                    self.checkInData.totalDiskSpaceBytes = unit.capacity;
                    self.checkInData.freeDiskSpace = Utility.convertToHumanReadable(self.checkInData.freeDiskSpaceBytes);
                    self.checkInData.totalDiskSpace = Utility.convertToHumanReadable(self.checkInData.totalDiskSpaceBytes);
                }
            }
            catch (err) {
                var errMsg = "checkinGetStorage() failed: " + JSON.stringify(err);
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            return self.checkinGetCpuLoad(callback);
        }, function (err) {
            self.checkInData.cpuUsage = -1;
            return self.checkinGetCpuLoad(callback);
        });
    };
    TizenPal.prototype.checkinGetDisplay = function (callback) {
        var self = this;
        console.log("---------- TizenPal.checkinGetDisplay");
        return tizen.systeminfo.getPropertyValue("DISPLAY", function (display) {
            try {
                if (display) {
                    var d = {};
                    d.model_name = self.checkInData.model;
                    d.port = 'display';
                    if (!d.resolutions) {
                        d.resolutions = [];
                    }
                    d.resolutions.push(display.resolutionWidth + "x" + display.resolutionHeight);
                    self.checkInData.displayInfos = [];
                    self.checkInData.displayInfos.push(d);
                }
            }
            catch (err) {
                var errMsg = "checkinGetDisplay() failed: " + JSON.stringify(err);
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            return self.checkinGetStorage(callback);
        }, function (err) {
            self.checkInData.cpuUsage = -1;
            return self.checkinGetStorage(callback);
        });
    };
    // checkinGetDateTimeInfo(callback: (ci?: iCheckin, err?: string) => void) {
    // }
    // checkinGetDUID(callback: (ci?: iCheckin, err?: string) => void) {
    //     // Get system ID information
    //     const self = this;
    //     // let duid = tizen.systeminfo.getCapability("http://tizen.org/system/tizenid");
    //     if (!self.player_name || self.player_name === '') {
    //         let pn = "tizen-" + self.wiredMac || self.wirelessMac;
    //         pn = pn.replace(/:/g, '');
    //         self.player_name = pn;
    //         self.checkInData.player_name = pn;
    //     }
    //     return self.checkinGetDisplay(callback);
    // }
    TizenPal.prototype.parseNetworkIface = function (infoIFace, macAddress, defaultName) {
        try {
            console.log("---------- TizenPal.parseNetworkIface");
            if (!this.checkInData.networkInterfaces[defaultName]) {
                this.checkInData.networkInterfaces[defaultName] = [];
            }
            var iFace = {};
            iFace.mac = macAddress;
            iFace.address = infoIFace.ipAddress;
            iFace.netmask = '';
            iFace.family = 'IPv4';
            this.checkInData.networkInterfaces[defaultName].push(iFace);
            iFace = {};
            iFace.mac = macAddress;
            if (infoIFace.ipv6Address && infoIFace.ipv6Address.length > 2) {
                iFace.address = infoIFace.ipv6Address;
            }
            else {
                iFace.address = '';
            }
            iFace.netmask = '';
            iFace.family = 'IPv6';
            this.checkInData.networkInterfaces[defaultName].push(iFace);
            this.checkInData.macAddress = this.macAddress(this.checkInData.networkInterfaces);
        }
        catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    };
    TizenPal.prototype.checkinGetWiFiNetworkInfo = function (callback) {
        var self = this;
        console.log("---------- TizenPal.checkinGetWiFiNetworkInfo");
        return tizen.systeminfo.getPropertyValue("WIFI_NETWORK", function (property) {
            // console.log(JSON.stringify(property));
            // {"status":"OFF","ssid":"","ipAddress":"","ipv6Address":"","macAddress":"","signalStrength":0}
            self.wirelessMac = property.macAddress || '';
            if (property.macAddress && property.macAddress !== "") {
                self.parseNetworkIface(property, self.wirelessMac, 'wlan0');
            }
            return self.checkinGetDisplay(callback);
        }, function (err) {
            console.log(JSON.stringify(err));
            return self.checkinGetDisplay(callback);
        });
    };
    TizenPal.prototype.checkinGetEthernetNetworkInfo = function (callback) {
        var self = this;
        try {
            console.log("---------- TizenPal.checkinGetEthernetNetworkInfo");
            return tizen.systeminfo.getPropertyValue("ETHERNET_NETWORK", function (property) {
                // console.log(JSON.stringify(property));
                // {"cable":"ATTACHED","status":"CONNECTED","ipAddress":"10.0.2.15","ipv6Address":"fec0::5054:ff:fe12:3456","macAddress":"52:54:00:12:34:56"}
                self.wiredMac = property.macAddress || '';
                if (property.macAddress && property.macAddress !== "") {
                    self.parseNetworkIface(property, self.wiredMac, 'eth0');
                }
                return self.checkinGetWiFiNetworkInfo(callback);
            }, function (err) {
                console.log(JSON.stringify(err));
                return self.checkinGetWiFiNetworkInfo(callback);
            });
        }
        catch (err) {
            console.error(err);
            return self.checkinGetWiFiNetworkInfo(callback);
        }
    };
    TizenPal.prototype.checkinGetNetworkInfo = function (callback) {
        var self = this;
        try {
            console.log("---------- TizenPal.checkinGetNetworkInfo");
            tizen.systeminfo.getPropertyValue("NETWORK", function (property) {
                // console.log(JSON.stringify(property));
                return self.checkinGetEthernetNetworkInfo(callback);
            }, function (err) {
                console.error(JSON.stringify(err));
                _klogger.error(err);
                return self.checkinGetEthernetNetworkInfo(callback);
            });
        }
        catch (err) {
            console.error(err);
            return self.checkinGetEthernetNetworkInfo(callback);
        }
    };
    TizenPal.prototype.checkinGetPlatformInfo = function (callback) {
        var self = this;
        try {
            console.log("---------- TizenPal.checkinGetPlatformInfo");
            tizen.systeminfo.getPropertyValue("BUILD", function (build) {
                try {
                    console.log(JSON.stringify(build));
                    self.checkInData.platform = build.manufacturer + " " + build.model + " (" + build.buildVersion + ")";
                    self.buildVersion = build.buildVersion;
                    self.manufacturer = build.manufacturer;
                    var appInfo = tizen.application.getCurrentApplication();
                    self.checkInData.version = appInfo.appInfo.version;
                    if (!self.screenSaverOff && webapis && webapis.appcommon && webapis.appcommon.setScreenSaver) {
                        try {
                            console.info('webapis.appcommon.setScreenSaver: SCREEN_SAVER_OFF');
                            webapis.appcommon.setScreenSaver(SCREEN_SAVER_OFF, function (data) {
                                console.info("Screen saver off: " + JSON.stringify(data));
                                self.screenSaverOff = true;
                            }, function (err) {
                                console.error(err);
                                _klogger.error(err);
                            });
                        }
                        catch (err) {
                            console.error(err);
                            _klogger.error(err);
                        }
                    }
                    else {
                        console.log("webapis.appcommon.setScreenSaver is not defined?");
                    }
                    if (!self.model) {
                        try {
                            if (webapis && webapis.productinfo && webapis.productinfo.getRealModel) {
                                self.model = webapis.productinfo.getRealModel();
                                self.firmware = webapis.productinfo.getFirmware();
                            }
                            else {
                                console.log("webapis.productinfo is not defined?");
                            }
                        }
                        catch (err) {
                            _klogger.error(err);
                        }
                    }
                    if (self.model) {
                        self.checkInData.model = self.model;
                        self.checkInData.osVersion = self.firmware;
                    }
                    try {
                        self.checkInData.osVersion = tizen.systeminfo.getCapability("http://tizen.org/feature/platform.version");
                        var webApiVer = tizen.systeminfo.getCapability("http://tizen.org/feature/platform.web.api.version");
                        if (webApiVer) {
                            self.checkInData.osVersion = self.checkInData.osVersion + "(" + webApiVer + ")";
                        }
                        self.checkInData.osArch = tizen.systeminfo.getCapability("http://tizen.org/feature/platform.core.cpu.arch");
                        self.checkInData.cpus = [];
                        self.checkInData.cpus.push({
                            model: tizen.systeminfo.getCapability("http://tizen.org/system/platform.processor"),
                            speed: tizen.systeminfo.getCapability("http://tizen.org/feature/platform.core.cpu.frequency")
                        });
                    }
                    catch (err) {
                        console.warn("checkinGetPlatformInfo(): " + JSON.stringify(err));
                        _klogger.warn("checkinGetPlatformInfo(): " + JSON.stringify(err));
                    }
                }
                catch (err) {
                    console.error(err);
                    _klogger.error(err);
                }
                return self.checkinGetNetworkInfo(callback);
            }, function (err) {
                console.log("checkinGetPlatformInfo: Error = " + JSON.stringify(err));
                return self.checkinGetNetworkInfo(callback);
            });
        }
        catch (err) {
            console.error(err);
            return self.checkinGetNetworkInfo(callback);
        }
    };
    TizenPal.prototype.checkinGetAppInfo = function (callback) {
        console.log('TizenPal.checkinGetAppInfo()');
        var self = this;
        try {
            self.checkinGetPlatformInfo(callback);
        }
        catch (err) {
            console.error("checkinGetAppInfo() failed: " + err);
            _klogger.error(err);
            self.checkinGetPlatformInfo(callback);
        }
    };
    TizenPal.prototype.checkinGetInfo = function (callback) {
        console.log('TizenPal.checkinGetInfo()');
        try {
            this.checkInData = {};
            this.checkInData.networkInterfaces = {};
            this.checkInData.player_name = this.player_name;
            this.checkInData.model = 'Tizen';
            if (this.provisioning && this.provisioning.tenant_token) {
                this.checkInData.tenant_token = this.provisioning.tenant_token;
            }
            this.checkInData.playerID = this.playerID;
            this.checkInData.token = this.provisioningToken;
            this.checkInData.displayInfos = [];
            var displayMode = window.outerWidth + "x" + window.outerHeight;
            var displayOrient = this.orientation + " degrees";
            this.checkInData.displayInfos.push({
                port: 'display',
                model_name: this.model,
                active: displayMode,
                preferred: displayMode,
                orientation: displayOrient,
                resolutions: [displayMode]
            });
            this.checkInData.displays = [];
            this.checkInData.displays.push({ port: 'display', resolution: displayMode, screenOrientation: displayOrient });
            this.checkinGetAppInfo(callback);
        }
        catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    };
    TizenPal.gotUsableStorage = function (device) {
        console.log('FOUND THE FILE I WANT');
    };
    TizenPal.checkUsableStorage = function (device, storages) {
        return tizen.filesystem.resolve(device.label, function (dir) {
            dir.listFiles(function (files) {
                for (var fdx = 0; fdx < files.length; fdx++) {
                    console.log("FILENAME: " + files[fdx].name);
                    if (files[fdx].isFile && files[fdx].name === 'korbyt-caching.json') {
                        return TizenPal.gotUsableStorage(device);
                    }
                }
                return TizenPal.gotStorages(storages);
            }, function (err) {
                console.error(JSON.stringify(err));
                return TizenPal.gotStorages(storages);
            });
        });
    };
    TizenPal.gotStorages = function (storages) {
        if (storages && storages.length > 0) {
            var device = storages.pop();
            console.log(JSON.stringify(device, null, 4));
            if (device.type === 'EXTERNAL' && device.state === 'MOUNTED') {
                return TizenPal.checkUsableStorage(device, storages);
            }
            else
                return TizenPal.gotStorages(storages);
        }
        else
            console.log('---- no more storage devices ----');
    };
    TizenPal.prototype.initialize = function (callback) {
        if (!this.isInitialized) {
            var self_1 = this;
            try {
                console.log('TizenPal.initialize()');
                tizen.filesystem.listStorages(TizenPal.gotStorages);
                self_1.provisioning = {
                    provisioned: false,
                    cms: '',
                    webService: '',
                    tenant_token: ''
                };
                if (window && window.b2bapis && window.b2bapis.b2bcontrol) {
                    b2bcontrol = window.b2bapis.b2bcontrol;
                }
                this.isInitialized = true;
            }
            catch (err) {
                console.error(err);
                _klogger.error(err);
            }
            self_1.loadSettings(function () {
                self_1.checkinGetInfo(function (ci, err) {
                    callback();
                });
            });
        }
        else {
            callback('PAL already initialized');
        }
    };
    TizenPal.prototype.enableDebugging = function () {
    };
    TizenPal.prototype.systemEventHandler = function (event) {
        _klogger.info({ systemEventMoniter: event });
        console.log("System Event [" + event.type + "] from " + event.source + ": " + JSON.stringify(event.data));
    };
    ;
    TizenPal.prototype.alertForCorruptedRemovableDrives = function (storages) {
        for (var i = 0; i < storages.length; i++) {
            if (storages[i].type != "EXTERNAL")
                continue;
            if (storages[i].state == "UNMOUNTABLE")
                console.warn("External drive " + storages[i].label + " is corrupted.");
            _klogger.warn("External drive " + storages[i].label + " is corrupted.");
        }
    };
    TizenPal.prototype.enableSystemEvents = function () {
        var self = this;
        try {
            try {
                tizen.filesystem.listStorages(this.alertForCorruptedRemovableDrives);
            }
            catch (err) {
                var errMsg = "register callback alertForCorruptedRemovableDrives() failed: " + JSON.stringify(err);
                console.error(errMsg);
                _klogger.error(errMsg);
            }
        }
        catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    };
    TizenPal.prototype.captureScreenShot = function (callbackPusher) {
        var self = this;
        try {
            if (!b2bcontrol) {
                return _klogger.warn('screenshot not supported (b2b missing)');
            }
            var onSuccess = function (screenShotPath) {
                console.log("[captureScreen] success : " + screenShotPath);
                tizen.filesystem.resolve(screenShotPath, function (fileObj) {
                    fileObj.openStream("r", function (fs) {
                        try {
                            var b64Img = fs.readBase64(fileObj.fileSize);
                            return callbackPusher(Utility.b64toBlob(b64Img, 'image/jpeg'));
                        }
                        catch (err) {
                            var errMsg = "captureScreenShot(" + screenShotPath + ")->readBytes(" + fileObj.fileSize + ") failed: " + JSON.stringify(err);
                            console.error(errMsg);
                            _klogger.error(errMsg);
                        }
                    }, function (err) {
                        var msg = "fileObj.openStream(\"r\"," + screenShotPath + ") Error: " + err;
                        console.log(msg);
                        _klogger.error(msg);
                    });
                }, function (err) {
                    var msg = "tizen.filesystem.resolve(" + screenShotPath + ") Error: " + err;
                    console.log(msg);
                    _klogger.error(msg);
                }, "rw");
            };
            var onError = function (error) {
                var msg = "[captureScreen] code :" + error.code + " error name: " + error.name + "  message " + error.message;
                console.error(msg);
                _klogger.error(msg);
            };
            console.log("[captureScreen] ");
            b2bcontrol.captureScreen(onSuccess, onError);
        }
        catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    };
    TizenPal.prototype.sendMessage = function (msgText, callback) {
        try {
            if (parent) {
                var msg = { to: 'host', message: msgText };
                parent.postMessage(JSON.stringify(msg), parent.location.toString());
            }
            else {
                _klogger.warn("no parent window found - failed postMessage: " + msgText);
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    TizenPal.prototype.setSleepSchedule = function (callback, schedule) {
        return main.notSupported('Sleep schedule is not supported.', callback);
    };
    TizenPal.prototype.upgradePlayer = function (callback, filename) {
        _klogger.warn('player upgrade is not supported');
        return callback();
    };
    TizenPal.prototype.restart = function (reason) {
        console.warn("========== APPLICATION.restart()");
        _klogger.warn("========== APPLICATION.restart()");
        if (reason) {
            main.LastRestartReason = reason;
        }
        location.reload(true);
    };
    TizenPal.prototype.reboot = function (reason) {
        try {
            console.warn("========== APPLICATION.reboot()");
            _klogger.warn("========== APPLICATION.reboot()");
            if (b2bcontrol) {
                var onSuccess = function (val) {
                    console.log("[rebootDevice] success : " + val);
                };
                var onError = function (error) {
                    var msg = "[rebootDevice] code :" + error.code + " error name: " + error.name + "  message " + error.message;
                    console.log(msg);
                    _klogger.error(msg);
                };
                if (reason) {
                    main.LastRestartReason = reason;
                }
                b2bcontrol.rebootDevice(onSuccess, onError);
            }
            else {
                location.reload(true);
            }
        }
        catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    };
    TizenPal.prototype.playerLoadingUrl = function (params, cachingEnabled) {
        if (!ResourceManager.FullDownloadPath) {
            cachingEnabled = main.cachingPlayerEnabled = false;
        }
        if (cachingEnabled) {
            return ResourceManager.FullDownloadPath + "/prt/"+ prtVersion + "/tizenplayer.html?p=" + params;
        }
        else {
            return main.pal.provisioning.cms + "/prt/"+ prtVersion +"/tizenplayer.html?p=" + params;
        }
    };
    TizenPal.prototype.IsNetworkAvailable = function (callback) {
        return tizen.systeminfo.getPropertyValue('NETWORK', function (networkInfo) {
            return callback(networkInfo && networkInfo.networkType !== 'NONE');
        });
    };
    TizenPal.__provisioningToken = null;
    return TizenPal;
}());
