/// <reference path="../../../Dev/src/jquery.d.ts" />
/// <reference path="../../../Dev/src/interfaces.ts" /> 
/// <xreference path="../../../Dev/src/enums.ts" /> 
/// <reference path="../../../Dev/src/Utility.ts" /> 
/// <xreference path="../../../Dev/src/logger.ts" /> 

declare var webapis: any;
declare var tizen: any;

interface Window {
    b2bapis: any
}

interface Screen {
    lockOrientation(b2bOrientVal: string);
    readonly availHeight: number;
    readonly availWidth: number;
    readonly colorDepth: number;
    readonly height: number;
    readonly orientation: ScreenOrientation;
    readonly pixelDepth: number;
    readonly width: number;
}

// declare var b2bapis: any;

interface SystemInfoProperty {
};

interface SystemInfoDisplay extends SystemInfoProperty {
    resolutionWidth: number;
    resolutionHeight: number;
    dotsPerInchWidth: number;
    dotsPerInchHeight: number;
    physicalWidth: number;
    physicalHeight: number;
    brightness: number;
};

interface SystemInfoBuild extends SystemInfoProperty {
    model: string;
    manufacturer: string;
    buildVersion: string;
    product: string;
};

const SCREEN_SAVER_OFF: number = 0;
const SCREEN_SAVER_ON: number = 1;
const KorbytSettingsFilename = "Korbyt_Settings.json";

let b2bcontrol: any = null;

let _globablPal: TizenPal = null;
function getKorbytPlayerPal() {
    if (!_globablPal) {
        _globablPal = new TizenPal();
    }
    return _globablPal;
}

class TizenPal {
    //  SystemInfoPropertyId= {
    // "BATTERY", "CPU", "STORAGE", "DISPLAY", "DEVICE_ORIENTATION", "BUILD", "LOCALE", "NETWORK", "WIFI_NETWORK", "ETHERNET_NETWORK",
    // "CELLULAR_NETWORK", "NET_PROXY_NETWORK", "SIM", "PERIPHERAL", "MEMORY", "VIDEOSOURCE", "CAMERA_FLASH", "ADS"};

    wiredMac: string;
    wirelessMac: string;
    player_name: string;
    playerID: number;
    wsUrl: string;
    cmsUrl: string;
    provisioning: iProvisioning;
    checkInData: iCheckin;
    forceSettings: boolean;
    showVisibleLogger: boolean = false;

    private buildVersion: string;
    private manufacturer: string;
    private serialNumber: string;
    private orientation: number = 0;
    private screenSaverOff: boolean = false;
    private model: string;
    private firmware: string;
    private isInitialized: boolean;
    private __checkInResults: any;

    private savedSettings: string;

    constructor() {
        this.isInitialized = false;
        this.screenSaverOff = false;
    }

    public set CheckInResults(cir: iCheckinResults) {
        if (cir) {
            this.__checkInResults = cir;
            _klogger.info(`localStorage.setItem ---  instructions === ${JSON.stringify(this.__checkInResults)}`);
            localStorage.setItem('instructions', JSON.stringify(this.__checkInResults));
        } else {
            localStorage.removeItem('instructions');
            _klogger.warn(`CheckInResults === null --- cleared`);
        }
    }

    public get CheckInResults() {
        try {
            if (!this.__checkInResults) {
                this.__checkInResults = localStorage.getItem('instructions');
                if (typeof this.__checkInResults === 'string') {
                    this.__checkInResults = JSON.parse(this.__checkInResults);
                }
            }
        } catch (err) {
            console.error(err);
        }
        return this.__checkInResults;
    }

    macAddress(ni: iNetworkInterfaces): iMacAddress[] {
        console.log("---------- TizenPal.macAddress");
        var macAddress: iMacAddress[] = [];
        for (var _interface in ni) {
            var map: boolean[] = [];
            for (var i = 0; i < ni[_interface].length; i++) {
                if (ni[_interface][i].address) {
                    const mac: string = ni[_interface][i].mac;
                    if (mac && mac !== "" && !map[mac]) {
                        macAddress.push(<iMacAddress>{ 'interface': _interface, 'address': mac });
                        map[mac] = true;
                    }
                }
            }
        }
        return macAddress;
    }

    private static __provisioningToken = null;
    get provisioningToken(): string {
        if (!TizenPal.__provisioningToken) {
            console.log("---------- TizenPal.provisioningToken");
            if (this.wiredMac) {
                TizenPal.__provisioningToken = Utility.encodeMac(this.wiredMac);
            }
        }
        return TizenPal.__provisioningToken;
    }

    provisioningDetails(callback: (details: iNameValuePair[]) => void): void {
        console.log("---------- TizenPal.provisioningDetails");
        const self = this;
        return self.checkinGetInfo(function (ci, err) {
            let details: iNameValuePair[] = [];
            try {
                let gotMac: boolean = false;
                details.push({ n: 'Model', v: ci.model });
                details.push({ n: 'Firmware', v: self.firmware });
                details.push({ n: 'Name', v: ci.player_name });
                details.push({ n: 'Software Version', v: ci.version });
                details.push({ n: 'Serial Number', v: ci.serialNumber });
                for (let i in ci.networkInterfaces) {
                    const nia: iNetworkInterface[] = ci.networkInterfaces[i];
                    for (let j in nia) {
                        const ni: iNetworkInterface = nia[j];
                        if (ni.address && ni.address != '') {
                            if (ni.mac && ni.mac != '') {
                                details.push({ n: 'MAC Address', v: ni.mac.replace(/:/g, '') });
                            }
                            if (!gotMac) {
                                gotMac = true;
                                details.push({ n: 'IP Address', v: ni.address });
                            }
                        }
                    }
                }
            } catch (err) {
                const errMsg = `provisioningDetails() failed: ${JSON.stringify(err)}`;
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            callback(details);
        });
    }

    checkProvisioning(webService: string, callback: () => void) {
        console.log(`---------- TizenPal.checkProvisioning ${webService}`);
        var self = this;
        return tizen.systeminfo.getPropertyValue('NETWORK', function (networkInfo) {
            try {
                if (networkInfo.networkType === 'NONE') {
                    _klogger.warn(`checkProvisioning() - network disconnected`);
                    return callback();
                }
                console.log(`TizenPal.checkProvisioning(${webService})`);
                var url = webService + `/v1/check?pt=${self.provisioningToken}`;
                console.info(url);
                try {

                    return $.ajax({
                        url: webService + `/v1/check?pt=${self.provisioningToken}`,
                        type: "GET",
                        dataType: "json",
                        success: (data) => {
                            // console.info("---------- checkProvisioning: " + JSON.stringify(data));
                            if (data.provisioned) {
                                self.provisioning = data;
                                return callback();
                            } else {
                                return callback();
                            }
                        },
                        error: (xhr, ajaxOptions, thrownError) => {
                            console.error("pal.checkProvisioning() Error - " + thrownError || ajaxOptions);
                            _klogger.error(thrownError);
                            return callback();
                        }
                    });
                } catch (err) {
                    _klogger.error(err);
                    return callback();
                }
            } catch (err) {
                _klogger.error(err);
                return callback();
            }
        });
    }

    parseSettingsFile(settings) {
        console.log(`---------- TizenPal.parseSettingsFile ${settings}`);
        try {
            if(!settings){
                settings = {};
            }
            this.wsUrl = settings.wsUrl || '';
            this.wiredMac = (settings.wiredMac || '').toLowerCase();
            this.wirelessMac = (settings.wirelessMac || '').toLowerCase();
            this.player_name = settings.player_name || '';
            this.playerID = settings.playerID || 0;
            this.provisioning = settings.provisioning || { provisioned: false };
            this.serialNumber = settings.serialNumber;
            this.orientation = settings.orientation;

            if (!this.CheckInResults) {
                if (typeof settings.checkInResults === 'string') {
                    settings.checkInResults = JSON.parse(settings.checkInResults);
                }
                if (settings.checkInResults) {
                    this.CheckInResults = settings.checkInResults;
                }
            }
            ResourceManager.FullDownloadPath = settings.FullDownloadPath;
            ResourceManager.PlayerSplashFullPath = settings.PlayerSplashFullPat;
        } catch (err) {
            console.error(err);
        }
    }

    FactoryReset() {
        this.checkInData = undefined;
        this.playerID = undefined;
        this.wsUrl = undefined;
        this.player_name = undefined;
        this.wiredMac = undefined;
        this.wirelessMac = undefined;
        this.provisioning = { provisioned: false, webService: '' };
        this.setOrientation(0);
        this.CheckInResults = undefined;
        localStorage.removeItem('KorbytSettings');
    }

    loadSettings(callback: () => void): void {
        console.log(`---------- TizenPal.loadSettings`);
        const self = this;
        try {
            let settingsObj = localStorage.getItem('KorbytSettings');
            try {
                settingsObj = JSON.parse(settingsObj);
            } catch (err) {
                console.error('################################################################################');
                console.error(err);
                console.error('################################################################################');
                console.error(settingsObj);
                console.error('################################################################################');
            }
            self.parseSettingsFile(settingsObj);
            return callback();
        } catch (err) {
            const errMsg = `checkProvisioning() failed: ${JSON.stringify(err)}`;
            console.error(errMsg);
            _klogger.error(errMsg);
            self.parseSettingsFile({});
            self.saveSettings(callback);
        }
    }

    saveSettings(callback: () => void): void {
        console.log(`---------- TizenPal.saveSettings`);
        var self = this;
        try {
            var settings: any = {
                wsUrl: self.wsUrl || '',
                wiredMac: self.wiredMac || '',
                wirelessMac: self.wirelessMac || '',
                player_name: self.player_name || '',
                playerID: self.playerID || 0,
                provisioning: self.provisioning || { provisioned: false, webService: '' },
                serialNumber: self.serialNumber,
                orientation: self.orientation,
                checkInResults: self.CheckInResults,
                FullDownloadPath: ResourceManager.FullDownloadPath,
                PlayerSplashFullPath: ResourceManager.PlayerSplashFullPath,
            };
            const textData: string = JSON.stringify(settings);
            localStorage.setItem('KorbytSettings', textData);

            return callback();
        } catch (err) {
            const errMsg = `saveSettings() failed: ${err}`;
            console.error(errMsg);
            _klogger.error(errMsg);
            return callback();
        }
    }

    checkinCompleted(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        try {
            console.log("---------- TizenPal.checkinCompleted");
            if (!self.player_name || self.player_name === '') {
                let pn = "tizen-" + self.wiredMac || self.wirelessMac;
                pn = pn.replace(/:/g, '');
                self.player_name = pn;
                self.checkInData.player_name = pn;
            }

            self.checkInData.freeMemBytes = tizen.systeminfo.getAvailableMemory();
            self.checkInData.totalMemBytes = tizen.systeminfo.getTotalMemory();
            self.checkInData.freemem = Utility.convertToHumanReadable(self.checkInData.freeMemBytes);
            self.checkInData.totalmem = Utility.convertToHumanReadable(self.checkInData.totalMemBytes);
            self.checkInData.volume = tizen.tvaudiocontrol.getVolume();
            self.checkInData.upgradeFilePattern = '^[a-zA-Z0-9-_ .]+[.]wgt$';

            // update current screen orientations
            if (window.screen && window.screen.orientation && typeof (window.screen.orientation.angle) === 'number') {
                this.orientation = window.screen.orientation.angle;
            }

            self.checkInData.properties = [
                {
                    name: "orientation",
                    displayName: "Orientation",
                    tooltip: "Display orientation",
                    type: "dropdown",
                    default: `${self.orientation}`,
                    values: [
                        ["0", "Landscape"],
                        ["90", "Portrait"]
                    ]
                }, {
                    name: "visualConsole",
                    displayName: "Visual Console",
                    tooltip: "Display console messages",
                    type: "dropdown",
                    default: `${self.showVisibleLogger ? 1 : 0}`,
                    values: [
                        ["0", "Disabled"],
                        ["1", "Enabled"]
                    ]
                }

            ];

            self.checkInData.capabilities = <iPlayerCapabilities>{
                screenShot: true,
                sleepSchedule: false,
                uploadLogs: true,
                remoteUpgrade: true,
                // videoFileExtensions: [".asf", ".wmv", ".divx", ".avi", ".mp4", ".m4v", ".mov", ".3gp", ".3g2", ".mkv", ".ts", ".trp", ".tp", ".mts", ".mpg", ".mpeg", ".dat", ".vob", ".webm"],
                // videoCodecs: ["H.264", "AVC", "MPEG-4", "DivX3.11", "DivX4", "DivX5", "DivX6", "XVID", , "MJpeg", "VC-1", "VP8", "VP9"],
                // audioFileExtensions: [".mp3", ".wav", ".ogg", ".wma"],
                // audioCodecs: ["Vorbis", "Opus", "AAC", "PCM", "MP3", "DTS", "LPCM", "ADPCM", "Dolby Digital", "Dolby Digital Plus", "WMA 9", "WMA", "AMR-NB", "AMR-WB"]
                verbs: [
                    "checkin",
                    "clear-cache",
                    "hide-QR",
                    // "hotkey",
                    "overlay",
                    "playerlog",
                    //"remove",
                    "rename",
                    "show-QR",
                    //"setresolution",
                    "screenshot",
                    "sleepTimes",
                    //"upgrade",
                    "urgent",
                    "volume"
                ],
                hotkeys: [
                    //"ARROWDOWN:Y:Rotate Display 180 degrees",
                    //"ARROWLEFT:Y:Rotate Display 270 degrees",
                    //"ARROWRIGHT:Y:Rotate Display 90 degrees",
                    //"ARROWUP:Y:Rotate Display 0 degrees",
                    "C:N:Clear Cache",
                    //"D:N:Toggle Debugger",
                    "I:N:Toggle Identity",
                    //"L:Y:Login",
                    //"Q:N:QR code toggle",
                    "R:Y:Reboot",
                    //"U:N:UI Debugger",
                    "X:Y:Player Exit",
                ]
            };

            try {
                if (b2bcontrol && !this.serialNumber) {
                    this.serialNumber = b2bcontrol.getSerialNumber();
                    self.checkInData.serialNumber = this.serialNumber;
                    return this.saveSettings(() => {
                        return callback(self.checkInData, null);
                    });
                }
                self.checkInData.serialNumber = this.serialNumber;
            } catch (e) {
                _klogger.log("[getSerialNumber] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
            }

            // self.checkInData.userAgent = navigator.userAgent;
            return callback(self.checkInData, null);
        } catch (err) {
            _klogger.error(err);
            return callback(self.checkInData, null);
        }
    }

    setOrientation(orient: number) {
        // const self = this;
        try {
            if (typeof (orient) === 'string') {
                orient = parseInt(orient);
            }
            if (orient === NaN) {
                orient = 0;
            }
            let b2bOrientVal: OrientationLockType;
            switch (orient) {
                case 270:
                case 90:
                    b2bOrientVal = 'portrait-primary';
                    break;
                case 180:
                default:
                    b2bOrientVal = 'landscape-primary';
                    break;
            }

            if (window.screen.orientation && window.screen.orientation.lock && typeof (window.screen.orientation.lock) === 'function') {
                window.screen.orientation.lock(b2bOrientVal);
            } else {
                // do this three times because Samsung said so.
                window.screen.lockOrientation(b2bOrientVal);
                window.screen.lockOrientation(b2bOrientVal);
                window.screen.lockOrientation(b2bOrientVal);
            }

            if (this.orientation != orient) {
                this.orientation = orient;
                main.nextCheckin = Date.now();
            }
        } catch (err) {
            _klogger.error(err);
        }
    }

    setSoundVolume(sndLevel: number) {
        try {
            if (sndLevel < 0) sndLevel = 0;
            if (sndLevel > 100) sndLevel = 100;
            if (sndLevel === 0) {
                tizen.tvaudiocontrol.setMute(true);
                console.log('TV audio muted');
            } else {
                tizen.tvaudiocontrol.setVolume(sndLevel);
                console.log(`TV audio volume = ${sndLevel}`);
            }
        } catch (err) {
            console.error(`setSoundVolume: ${err}`);
            _klogger.error(err);
        }
    }

    getCpuLoad(callback: (cpuLoad: number) => any) {
        return tizen.systeminfo.getPropertyValue("CPU", (cpu) => {
            return callback(cpu.load);
        }, (err) => {
            _klogger.error(err);
            return callback(undefined);
        });
    }

    getMemoryLoad(callback: (memLoad: number) => any) {
        return callback((tizen.systeminfo.getAvailableMemory() / tizen.systeminfo.getTotalMemory()) * 100);
    }

    getStorageLoad(callback: (storageLoad: number) => any) {
        return tizen.systeminfo.getPropertyValue("STORAGE", (storage) => {
            try {
                if (storage && storage.units && storage.units.length > 0) {
                    var unit = storage.units[0];
                    return callback((unit.availableCapacity / unit.capacity) * 100);
                }
            } catch (err) {
                _klogger.error(err);
            }
            return callback(undefined);
        }, (err) => {
            _klogger.error(err);
            return callback(undefined);
        });
    }

    checkinGetCpuLoad(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        console.log("---------- TizenPal.checkinGetCpuLoad");
        return tizen.systeminfo.getPropertyValue("CPU", (cpu) => {
            self.checkInData.cpuUsage = cpu.load;
            return self.checkinCompleted(callback);
        }, (err) => {
            console.error(`checkinGetCpuLoad: ${err}`);
            _klogger.error(err);
            return self.checkinCompleted(callback);
        });
    };

    checkinGetStorage(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        console.log("---------- TizenPal.checkinGetStorage");
        return tizen.systeminfo.getPropertyValue("STORAGE", (storage) => {
            try {
                if (storage && storage.units && storage.units.length > 0) {
                    var unit = storage.units[0];
                    self.checkInData.freeDiskSpaceBytes = unit.availableCapacity;
                    self.checkInData.totalDiskSpaceBytes = unit.capacity;
                    self.checkInData.freeDiskSpace = Utility.convertToHumanReadable(self.checkInData.freeDiskSpaceBytes);
                    self.checkInData.totalDiskSpace = Utility.convertToHumanReadable(self.checkInData.totalDiskSpaceBytes);
                }
            } catch (err) {
                const errMsg = `checkinGetStorage() failed: ${JSON.stringify(err)}`;
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            return self.checkinGetCpuLoad(callback);
        }, (err) => {
            self.checkInData.cpuUsage = -1;
            return self.checkinGetCpuLoad(callback);
        });
    }

    checkinGetDisplay(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        console.log("---------- TizenPal.checkinGetDisplay");
        return tizen.systeminfo.getPropertyValue("DISPLAY", (display: SystemInfoDisplay) => {
            try {
                if (display) {
                    let d: iDisplayInfo = <iDisplayInfo>{};
                    d.model_name = self.checkInData.model;
                    d.port = 'display';
                    if (!d.resolutions) {
                        d.resolutions = [];
                    }
                    d.resolutions.push(`${display.resolutionWidth}x${display.resolutionHeight}`);
                    self.checkInData.displayInfos = [];
                    self.checkInData.displayInfos.push(d);
                }
            } catch (err) {
                const errMsg = `checkinGetDisplay() failed: ${JSON.stringify(err)}`;
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            return self.checkinGetStorage(callback);
        }, (err) => {
            self.checkInData.cpuUsage = -1;
            return self.checkinGetStorage(callback);
        });
    }

    // checkinGetDateTimeInfo(callback: (ci?: iCheckin, err?: string) => void) {

    // }

    // checkinGetDUID(callback: (ci?: iCheckin, err?: string) => void) {
    //     // Get system ID information
    //     const self = this;
    //     // let duid = tizen.systeminfo.getCapability("http://tizen.org/system/tizenid");
    //     if (!self.player_name || self.player_name === '') {
    //         let pn = "tizen-" + self.wiredMac || self.wirelessMac;
    //         pn = pn.replace(/:/g, '');
    //         self.player_name = pn;
    //         self.checkInData.player_name = pn;
    //     }
    //     return self.checkinGetDisplay(callback);
    // }

    private parseNetworkIface(infoIFace: any, macAddress: string, defaultName: string) {
        try {
            console.log("---------- TizenPal.parseNetworkIface");

            if (!this.checkInData.networkInterfaces[defaultName]) {
                this.checkInData.networkInterfaces[defaultName] = [];
            }

            let iFace: iNetworkInterface = <iNetworkInterface>{};
            iFace.mac = macAddress;
            iFace.address = infoIFace.ipAddress;
            iFace.netmask = '';
            iFace.family = 'IPv4';

            this.checkInData.networkInterfaces[defaultName].push(iFace);

            iFace = <iNetworkInterface>{};
            iFace.mac = macAddress;
            if (infoIFace.ipv6Address && infoIFace.ipv6Address.length > 2) {
                iFace.address = infoIFace.ipv6Address;
            } else {
                iFace.address = '';
            }
            iFace.netmask = '';
            iFace.family = 'IPv6';
            this.checkInData.networkInterfaces[defaultName].push(iFace);

            this.checkInData.macAddress = this.macAddress(this.checkInData.networkInterfaces);
        } catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    }

    checkinGetWiFiNetworkInfo(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        console.log("---------- TizenPal.checkinGetWiFiNetworkInfo");
        return tizen.systeminfo.getPropertyValue("WIFI_NETWORK", (property) => {
            // console.log(JSON.stringify(property));
            // {"status":"OFF","ssid":"","ipAddress":"","ipv6Address":"","macAddress":"","signalStrength":0}
            self.wirelessMac = property.macAddress || '';
            if (property.macAddress && property.macAddress !== "") {
                self.parseNetworkIface(property, self.wirelessMac, 'wlan0');
            }
            return self.checkinGetDisplay(callback);
        }, (err) => {
            console.log(JSON.stringify(err));
            return self.checkinGetDisplay(callback);
        });
    }

    checkinGetEthernetNetworkInfo(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        try {
            console.log("---------- TizenPal.checkinGetEthernetNetworkInfo");
            return tizen.systeminfo.getPropertyValue("ETHERNET_NETWORK", (property) => {
                // console.log(JSON.stringify(property));
                // {"cable":"ATTACHED","status":"CONNECTED","ipAddress":"10.0.2.15","ipv6Address":"fec0::5054:ff:fe12:3456","macAddress":"52:54:00:12:34:56"}
                self.wiredMac = property.macAddress || '';
                if (property.macAddress && property.macAddress !== "") {
                    self.parseNetworkIface(property, self.wiredMac, 'eth0');
                }
                return self.checkinGetWiFiNetworkInfo(callback);
            }, (err) => {
                console.log(JSON.stringify(err));
                return self.checkinGetWiFiNetworkInfo(callback);
            });
        } catch (err) {
            console.error(err);
            return self.checkinGetWiFiNetworkInfo(callback);
        }
    }

    checkinGetNetworkInfo(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        try {
            console.log("---------- TizenPal.checkinGetNetworkInfo");
            tizen.systeminfo.getPropertyValue("NETWORK", (property) => {
                // console.log(JSON.stringify(property));
                return self.checkinGetEthernetNetworkInfo(callback);
            }, (err) => {
                console.error(JSON.stringify(err));
                _klogger.error(err);
                return self.checkinGetEthernetNetworkInfo(callback);
            });
        } catch (err) {
            console.error(err);
            return self.checkinGetEthernetNetworkInfo(callback);
        }
    }

    checkinGetPlatformInfo(callback: (ci?: iCheckin, err?: string) => void) {
        const self = this;
        try {
            console.log("---------- TizenPal.checkinGetPlatformInfo");
            tizen.systeminfo.getPropertyValue("BUILD", (build: SystemInfoBuild) => {
                try {
                    console.log(JSON.stringify(build));
                    self.checkInData.platform = `${build.manufacturer} ${build.model} (${build.buildVersion})`;
                    self.buildVersion = build.buildVersion;
                    self.manufacturer = build.manufacturer;
                    let appInfo = tizen.application.getCurrentApplication();
                    self.checkInData.version = appInfo.appInfo.version;
                    if (!self.screenSaverOff && webapis && webapis.appcommon && webapis.appcommon.setScreenSaver) {
                        try {
                            console.info('webapis.appcommon.setScreenSaver: SCREEN_SAVER_OFF');
                            webapis.appcommon.setScreenSaver(SCREEN_SAVER_OFF, (data) => {
                                console.info("Screen saver off: " + JSON.stringify(data));
                                self.screenSaverOff = true;
                            }, (err) => {
                                console.error(err);
                                _klogger.error(err);
                            });
                        } catch (err) {
                            console.error(err);
                            _klogger.error(err);
                        }
                    } else {
                        console.log("webapis.appcommon.setScreenSaver is not defined?");
                    }

                    if (!self.model) {
                        try {
                            if (webapis && webapis.productinfo && webapis.productinfo.getRealModel) {
                                self.model = webapis.productinfo.getRealModel();
                                self.firmware = webapis.productinfo.getFirmware();
                            } else {
                                console.log("webapis.productinfo is not defined?");
                            }
                        } catch (err) {
                            _klogger.error(err);
                        }
                    }
                    if (self.model) {
                        self.checkInData.model = self.model;
                        self.checkInData.osVersion = self.firmware;
                    }

                    try {
                        self.checkInData.osVersion = tizen.systeminfo.getCapability("http://tizen.org/feature/platform.version");
                        let webApiVer = tizen.systeminfo.getCapability("http://tizen.org/feature/platform.web.api.version");
                        if (webApiVer) {
                            self.checkInData.osVersion = `${self.checkInData.osVersion}(${webApiVer})`;
                        }
                        self.checkInData.osArch = tizen.systeminfo.getCapability("http://tizen.org/feature/platform.core.cpu.arch");
                        self.checkInData.cpus = [];
                        self.checkInData.cpus.push(<iCpu>{
                            model: tizen.systeminfo.getCapability("http://tizen.org/system/platform.processor"),
                            speed: tizen.systeminfo.getCapability("http://tizen.org/feature/platform.core.cpu.frequency")
                        });
                    } catch (err) {
                        console.warn(`checkinGetPlatformInfo(): ${JSON.stringify(err)}`);
                        _klogger.warn(`checkinGetPlatformInfo(): ${JSON.stringify(err)}`);
                    }
                } catch (err) {
                    console.error(err);
                    _klogger.error(err);
                }
                return self.checkinGetNetworkInfo(callback);
            }, (err) => {
                console.log(`checkinGetPlatformInfo: Error = ${JSON.stringify(err)}`);
                return self.checkinGetNetworkInfo(callback);
            });
        } catch (err) {
            console.error(err);
            return self.checkinGetNetworkInfo(callback);
        }
    }

    checkinGetAppInfo(callback: (ci?: iCheckin, err?: string) => void): void {
        console.log('TizenPal.checkinGetAppInfo()');
        const self = this;
        try {
            // // Get IPK type app information from appinfo.json.
            // var custom = new Custom();
            // custom.Signage.getApplicationInfo(
            //     function successCallback(successObject) {
            //         console.log('Data from appinfo.json : ' + JSON.stringify(successObject, null, 4));
            //         self.checkInData.version = successObject.version;
            //         self.checkinGetPlatformInfo(callback);
            //     },
            //     function failureCallback(failureObject) {
            //         console.error(failureObject);
            //         logger.error('[' + failureObject.errorCode + '] ' + failureObject.errorText)
            //         self.checkInData.version = '--.--.--';
            //         self.checkinGetPlatformInfo(callback);
            //     });
            self.checkinGetPlatformInfo(callback);
        } catch (err) {
            console.error(`checkinGetAppInfo() failed: ${err}`);
            _klogger.error(err);
            self.checkinGetPlatformInfo(callback);
        }
    }

    checkinGetInfo(callback: (ci?: iCheckin, err?: string) => void): void {
        console.log('TizenPal.checkinGetInfo()');
        try {
            this.checkInData = <iCheckin>{};
            this.checkInData.networkInterfaces = <iNetworkInterfaces>{};
            this.checkInData.player_name = this.player_name;
            this.checkInData.model = 'Tizen';
            if (this.provisioning && this.provisioning.tenant_token) {
                this.checkInData.tenant_token = this.provisioning.tenant_token;
            }

            this.checkInData.playerID = this.playerID;
            this.checkInData.token = this.provisioningToken;
            this.checkInData.displayInfos = [];
            const displayMode: string = `${window.outerWidth}x${window.outerHeight}`;
            const displayOrient: string = `${this.orientation} degrees`;
            this.checkInData.displayInfos.push(<iDisplayInfo>{
                port: 'display',
                model_name: this.model,
                active: displayMode,
                preferred: displayMode,
                orientation: displayOrient,
                resolutions: [displayMode]
            });
            this.checkInData.displays = [];
            this.checkInData.displays.push(<iDisplay>{ port: 'display', resolution: displayMode, screenOrientation: displayOrient });

            this.checkinGetAppInfo(callback);
        } catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    }

    static gotUsableStorage(device) {
        console.log('FOUND THE FILE I WANT');
    }

    static checkUsableStorage(device, storages) {
        return tizen.filesystem.resolve(device.label, (dir) => {
            dir.listFiles(
                (files) => {
                    for (let fdx = 0; fdx < files.length; fdx++) {
                        console.log(`FILENAME: ${files[fdx].name}`);
                        if (files[fdx].isFile && files[fdx].name === 'korbyt-caching.json') {
                            return TizenPal.gotUsableStorage(device);
                        }
                    }

                    return TizenPal.gotStorages(storages);
                },
                (err) => {
                    console.error(JSON.stringify(err));
                    return TizenPal.gotStorages(storages);
                });
        });
    }

    static gotStorages(storages) {
        if (storages && storages.length > 0) {
            let device = storages.pop();
            console.log(JSON.stringify(device, null, 4));
            if (device.type === 'EXTERNAL' && device.state === 'MOUNTED') {
                return TizenPal.checkUsableStorage(device, storages);
            } else
                return TizenPal.gotStorages(storages);
        }
        else
            console.log('---- no more storage devices ----');
    }

    initialize(callback: (err?: string) => void): void {
        if (!this.isInitialized) {
            const self = this;
            try {
                console.log('TizenPal.initialize()');

                tizen.filesystem.listStorages(TizenPal.gotStorages);

                self.provisioning = <iProvisioning>{
                    provisioned: false,
                    cms: '',
                    webService: '',
                    tenant_token: ''
                };

                if (window && window.b2bapis && window.b2bapis.b2bcontrol) {
                    b2bcontrol = window.b2bapis.b2bcontrol;
                }

                this.isInitialized = true;
            } catch (err) {
                console.error(err);
                _klogger.error(err);
            }

            self.loadSettings(function () {
                self.checkinGetInfo((ci: iCheckin, err?: string) => {
                    callback();
                });
            });
        } else {
            callback('PAL already initialized');
        }
    }

    enableDebugging() {
        // var s = document.createElement('script');
        // s.src = 'https://unpkg.com/dts-gen/bin/browser-bundle.js';
        // document.body.appendChild(s);
        // var configuration = new Configuration();
        // var options = { enabled: true };
        // configuration.debug(
        //     function successCb() {
        //         console.log("Succeeded to enable the debug mode");
        //     },
        //     function failureCb() {
        //         console.log("Failed to enable the debug mode");
        //     },
        //     options);
    }

    systemEventHandler(event) {
        _klogger.info({ systemEventMoniter: event });
        console.log(`System Event [${event.type}] from ${event.source}: ${JSON.stringify(event.data)}`);
    };

    alertForCorruptedRemovableDrives(storages) {
        for (var i = 0; i < storages.length; i++) {
            if (storages[i].type != "EXTERNAL")
                continue;
            if (storages[i].state == "UNMOUNTABLE")
                console.warn("External drive " + storages[i].label + " is corrupted.");
            _klogger.warn("External drive " + storages[i].label + " is corrupted.");
        }
    }

    enableSystemEvents() {
        const self = this;
        try {
            try {
                tizen.filesystem.listStorages(this.alertForCorruptedRemovableDrives);
            } catch (err) {
                const errMsg = `register callback alertForCorruptedRemovableDrives() failed: ${JSON.stringify(err)}`;
                console.error(errMsg);
                _klogger.error(errMsg);
            }
            // var signage = new Signage();
            // signage.registerSystemMonitor(() => {
            //     console.log('enableSystemEvents()->registerSystemMonitor() success');
            // }, (cbObject) => {
            //     logger.error(`enableSystemEvents()->registerSystemMonitor() Error Code [${cbObject.errorCode}]:  ${cbObject.errorText}`);
            // }, {
            //         monitorConfiguration: {
            //             fan: true,
            //             lamp: true,
            //             screen: true,
            //             signal: true,
            //             temperature: true
            //         },
            //         eventHandler: (eventData) => {
            //             self.systemEventHandler(eventData);
            //         }
            //     });
        } catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    }

    captureScreenShot(callbackPusher: (screenShot: any) => any) {
        const self = this;
        try {
            if (!b2bcontrol) {
                return _klogger.warn('screenshot not supported (b2b missing)');
            }

            let onSuccess = function (screenShotPath) {
                console.log("[captureScreen] success : " + screenShotPath);
                tizen.filesystem.resolve(screenShotPath,
                    (fileObj) => {
                        fileObj.openStream("r",
                            (fs) => {
                                try {
                                    let b64Img = fs.readBase64(fileObj.fileSize);
                                    return callbackPusher(Utility.b64toBlob(b64Img, 'image/jpeg'));
                                } catch (err) {
                                    const errMsg = `captureScreenShot(${screenShotPath})->readBytes(${fileObj.fileSize}) failed: ${JSON.stringify(err)}`;
                                    console.error(errMsg);
                                    _klogger.error(errMsg);
                                }
                            },
                            (err) => {
                                const msg: string = `fileObj.openStream("r",${screenShotPath}) Error: ${err}`;
                                console.log(msg);
                                _klogger.error(msg);
                            });
                    }, (err) => {
                        const msg: string = `tizen.filesystem.resolve(${screenShotPath}) Error: ${err}`;
                        console.log(msg);
                        _klogger.error(msg);
                    }, "rw");
            };

            let onError = function (error) {
                const msg: string = "[captureScreen] code :" + error.code + " error name: " + error.name + "  message " + error.message;
                console.error(msg);
                _klogger.error(msg);
            };

            console.log("[captureScreen] ");
            b2bcontrol.captureScreen(onSuccess, onError);
        } catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    }

    sendMessage(msgText: any, callback?: (response?: any) => void): void {
        try {
            if (parent) {
                const msg: any = { to: 'host', message: msgText };
                parent.postMessage(JSON.stringify(msg), parent.location.toString());
            } else {
                _klogger.warn(`no parent window found - failed postMessage: ${msgText}`);
            }
        } catch (err) {
            _klogger.error(err);
        }
    }

    setSleepSchedule(callback: () => any, schedule: string) {
        return main.notSupported('Sleep schedule is not supported.', callback);
    }

    upgradePlayer(callback: () => any, filename: string) {
        _klogger.warn('player upgrade is not supported');
        return callback();
    }

    restart(reason: string) {
        console.warn("========== APPLICATION.restart()");
        _klogger.warn("========== APPLICATION.restart()");
        if (reason) {
            main.LastRestartReason = reason;
        }
        location.reload(true);
    }

    reboot(reason: string) {
        try {
            console.warn("========== APPLICATION.reboot()");
            _klogger.warn("========== APPLICATION.reboot()");
            if (b2bcontrol) {
                let onSuccess = function (val) {
                    console.log("[rebootDevice] success : " + val);
                };
                let onError = function (error) {
                    const msg: string = "[rebootDevice] code :" + error.code + " error name: " + error.name + "  message " + error.message;
                    console.log(msg);
                    _klogger.error(msg);
                };
                if (reason) {
                    main.LastRestartReason = reason;
                }
                b2bcontrol.rebootDevice(onSuccess, onError);
            } else {
                location.reload(true);
            }
        } catch (err) {
            console.error(err);
            _klogger.error(err);
        }
    }

    playerLoadingUrl(params: string, cachingEnabled: boolean): string {
        if (!ResourceManager.FullDownloadPath) {
            cachingEnabled = main.cachingPlayerEnabled = false;
        }
        if (cachingEnabled) {
            return `${ResourceManager.FullDownloadPath}/tizencached.html?p=${params}`;
        } else {
            return `${main.pal.provisioning.cms}/prt/v3.8/tizenplayer.html?p=${params}`;
        }
    }

    IsNetworkAvailable(callback: (online: boolean) => any) {
        return tizen.systeminfo.getPropertyValue('NETWORK', (networkInfo) => {
            return callback(networkInfo && networkInfo.networkType !== 'NONE');
        });
    }
}